# -*- coding:utf-8 -*-
# !/usr/bin/env python
"""
Date: 2024/6/27 22:20
Desc: 上海证券交易所-产品-股票期权-每日统计
https://www.sse.com.cn/assortment/options/date/
深圳证券交易所-市场数据-期权数据-日度概况
https://investor.szse.cn/market/option/day/index.html
"""

import pandas as pd
import requests


def option_daily_stats_sse(date: str = "20240626") -> pd.DataFrame:
    """
    上海证券交易所-产品-股票期权-每日统计
    https://www.sse.com.cn/assortment/options/date/
    :param date: 交易日
    :type date: str
    :return: 每日统计
    :rtype: pandas.DataFrame
    """
    url = "http://query.sse.com.cn/commonQuery.do"
    params = {
        "isPagination": "false",
        "sqlId": "COMMON_SSE_ZQPZ_YSP_QQ_SJTJ_MRTJ_CX",
        "tradeDate": date,
        "_": "1652877575590",
    }
    headers = {
        "Accept": "*/*",
        "Accept-Encoding": "gzip, deflate",
        "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
        "Cache-Control": "no-cache",
        "Connection": "keep-alive",
        "Host": "query.sse.com.cn",
        "Pragma": "no-cache",
        "Referer": "https://www.sse.com.cn/",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/101.0.4951.67 Safari/537.36",
    }
    r = requests.get(url, params=params, headers=headers)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["result"])
    temp_df.rename(
        columns={
            "CONTRACT_VOLUME": "合约数量",
            "CALL_VOLUME": "认购成交量",
            "LEAVES_QTY": "未平仓合约总数",
            "CP_RATE": "认沽/认购",
            "PUT_VOLUME": "认沽成交量",
            "TRADE_DATE": "交易日",
            "TOTAL_MONEY": "总成交额",
            "TOTAL_VOLUME": "总成交量",
            "SECURITY_CODE": "合约标的代码",
            "LEAVES_CALL_QTY": "未平仓认购合约数",
            "LEAVES_PUT_QTY": "未平仓认沽合约数",
            "SECURITY_ABBR": "合约标的名称",
        },
        inplace=True,
    )
    temp_df = temp_df[
        [
            "合约标的代码",
            "合约标的名称",
            "合约数量",
            "总成交额",
            "总成交量",
            "认购成交量",
            "认沽成交量",
            "认沽/认购",
            "未平仓合约总数",
            "未平仓认购合约数",
            "未平仓认沽合约数",
            "交易日",
        ]
    ]
    temp_df["交易日"] = pd.to_datetime(temp_df["交易日"], errors="coerce").dt.date
    for item in temp_df.columns[2:-1]:
        temp_df[item] = temp_df[item].str.replace(",", "")
        temp_df[item] = pd.to_numeric(temp_df[item], errors="coerce")
    return temp_df


def option_daily_stats_szse(date: str = "20240626") -> pd.DataFrame:
    """
    深圳证券交易所-市场数据-期权数据-日度概况
    https://investor.szse.cn/market/option/day/index.html
    :param date: 交易日
    :type date: str
    :return: 每日统计
    :rtype: pandas.DataFrame
    """
    url = "https://investor.szse.cn/api/report/ShowReport/data"
    params = {
        "SHOWTYPE": "JSON",
        "CATALOGID": "ysprdzb",
        "TABKEY": "tab1",
        "txtQueryDate": "-".join([date[:4], date[4:6], date[6:]]),
        "random": "0.0652692406565949",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json[0]["data"])
    temp_df.rename(
        columns={
            "bddm": "合约标的代码",
            "bdmc": "合约标的名称",
            "cjl": "成交量",
            "rccjl": "认购成交量",
            "rpcjl": "认沽成交量",
            "rcrpccb": "认沽/认购持仓比",
            "wpchyzs": "未平仓合约总数",
            "wpcrchys": "未平仓认购合约数",
            "wpcrphys": "未平仓认沽合约数",
        },
        inplace=True,
    )
    temp_df = temp_df[
        [
            "合约标的代码",
            "合约标的名称",
            "成交量",
            "认购成交量",
            "认沽成交量",
            "认沽/认购持仓比",
            "未平仓合约总数",
            "未平仓认购合约数",
            "未平仓认沽合约数",
        ]
    ]
    temp_df["交易日"] = "-".join([date[:4], date[4:6], date[6:]])
    temp_df["交易日"] = pd.to_datetime(temp_df["交易日"], errors="coerce").dt.date
    for item in temp_df.columns[2:-1]:
        temp_df[item] = temp_df[item].str.replace(",", "")
        temp_df[item] = pd.to_numeric(temp_df[item], errors="coerce")
    return temp_df


if __name__ == "__main__":
    option_daily_stats_sse_df = option_daily_stats_sse(date="20240626")
    print(option_daily_stats_sse_df)

    option_daily_stats_szse_df = option_daily_stats_szse(date="20240626")
    print(option_daily_stats_szse_df)



# -*- coding:utf-8 -*-
# !/usr/bin/env python
"""
Date: 2022/1/25 10:20
Desc: 东方财富网-数据中心-特色数据-期权折溢价
https://data.eastmoney.com/other/premium.html
"""
import requests
import pandas as pd


def option_premium_analysis_em() -> pd.DataFrame:
    """
    东方财富网-数据中心-特色数据-期权折溢价
    https://data.eastmoney.com/other/premium.html
    :return: 期权折溢价
    :rtype: pandas.DataFrame
    """
    url = "https://push2.eastmoney.com/api/qt/clist/get"
    params = {
        'fid': 'f250',
        'po': '1',
        'pz': '5000',
        'pn': '1',
        'np': '1',
        'fltt': '2',
        'invt': '2',
        'ut': 'b2884a393a59ad64002292a3e90d46a5',
        'fields': 'f1,f2,f3,f12,f13,f14,f161,f250,f330,f331,f332,f333,f334,f335,f337,f301,f152',
        'fs': 'm:10'
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["data"]["diff"])
    temp_df.columns = [
        '-',
        '最新价',
        '涨跌幅',
        '期权代码',
        '-',
        '期权名称',
        '-',
        '行权价',
        '折溢价率',
        '到期日',
        '-',
        '-',
        '-',
        '标的名称',
        '标的最新价',
        '标的涨跌幅',
        '盈亏平衡价',
    ]
    temp_df = temp_df[[
        '期权代码',
        '期权名称',
        '最新价',
        '涨跌幅',
        '行权价',
        '折溢价率',
        '标的名称',
        '标的最新价',
        '标的涨跌幅',
        '盈亏平衡价',
        '到期日',
    ]]
    temp_df['最新价'] = pd.to_numeric(temp_df['最新价'], errors="coerce")
    temp_df['涨跌幅'] = pd.to_numeric(temp_df['涨跌幅'], errors="coerce")
    temp_df['行权价'] = pd.to_numeric(temp_df['行权价'], errors="coerce")
    temp_df['折溢价率'] = pd.to_numeric(temp_df['折溢价率'], errors="coerce")
    temp_df['标的最新价'] = pd.to_numeric(temp_df['标的最新价'], errors="coerce")
    temp_df['标的涨跌幅'] = pd.to_numeric(temp_df['标的涨跌幅'], errors="coerce")
    temp_df['盈亏平衡价'] = pd.to_numeric(temp_df['盈亏平衡价'], errors="coerce")
    temp_df['到期日'] = pd.to_datetime(temp_df['到期日'].astype(str)).dt.date
    return temp_df


if __name__ == "__main__":
    option_premium_analysis_em_df = option_premium_analysis_em()
    print(option_premium_analysis_em_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2023/4/5 22:05
Desc: 乐咕乐股-基金仓位
https://legulegu.com/stockdata/fund-position/pos-stock
"""
import pandas as pd
import requests

from byteapp_ak.stock_feature.stock_a_indicator import get_token_lg, get_cookie_csrf


def fund_stock_position_lg() -> pd.DataFrame:
    """
    乐咕乐股-基金仓位-股票型基金仓位
    https://legulegu.com/stockdata/fund-position/pos-stock
    :return: 股票型基金仓位
    :rtype: pandas.DataFrame
    """
    url = "https://legulegu.com/api/stockdata/fund-position"
    token = get_token_lg()
    params = {"token": token, "type": "pos_stock", "category": "总仓位", "marketId": "5"}
    r = requests.get(
        url,
        params=params,
        **get_cookie_csrf(url="https://legulegu.com/stockdata/fund-position/pos-stock")
    )
    data_json = r.json()
    temp_df = pd.DataFrame(data_json)
    temp_df["date"] = pd.to_datetime(temp_df["date"]).dt.date
    temp_df = temp_df[
        [
            "date",
            "close",
            "position",
        ]
    ]
    temp_df['date'] = pd.to_datetime(temp_df['date']).dt.date
    temp_df["close"] = pd.to_numeric(temp_df["close"], errors="coerce")
    temp_df["position"] = pd.to_numeric(temp_df["position"], errors="coerce")
    return temp_df


def fund_balance_position_lg() -> pd.DataFrame:
    """
    乐咕乐股-基金仓位-平衡混合型基金仓位
    https://legulegu.com/stockdata/fund-position/pos-pingheng
    :return: 平衡混合型基金仓位
    :rtype: pandas.DataFrame
    """
    url = "https://legulegu.com/api/stockdata/fund-position"
    token = get_token_lg()
    params = {
        "token": token,
        "type": "pos_pingheng",
        "category": "总仓位",
        "marketId": "5",
    }
    r = requests.get(
        url,
        params=params,
        **get_cookie_csrf(
            url="https://legulegu.com/stockdata/fund-position/pos-pingheng"
        )
    )
    data_json = r.json()
    temp_df = pd.DataFrame(data_json)
    temp_df["date"] = pd.to_datetime(temp_df["date"]).dt.date
    temp_df = temp_df[
        [
            "date",
            "close",
            "position",
        ]
    ]
    temp_df['date'] = pd.to_datetime(temp_df['date']).dt.date
    temp_df["close"] = pd.to_numeric(temp_df["close"], errors="coerce")
    temp_df["position"] = pd.to_numeric(temp_df["position"], errors="coerce")
    return temp_df


def fund_linghuo_position_lg() -> pd.DataFrame:
    """
    乐咕乐股-基金仓位-灵活配置型基金仓位
    https://legulegu.com/stockdata/fund-position/pos-linghuo
    :return: 灵活配置型基金仓位
    :rtype: pandas.DataFrame
    """
    url = "https://legulegu.com/api/stockdata/fund-position"
    token = get_token_lg()
    params = {"token": token, "type": "pos_linghuo", "category": "总仓位", "marketId": "5"}
    r = requests.get(
        url,
        params=params,
        **get_cookie_csrf(
            url="https://legulegu.com/stockdata/fund-position/pos-linghuo"
        )
    )
    data_json = r.json()
    temp_df = pd.DataFrame(data_json)
    temp_df["date"] = pd.to_datetime(temp_df["date"]).dt.date
    temp_df = temp_df[
        [
            "date",
            "close",
            "position",
        ]
    ]
    temp_df['date'] = pd.to_datetime(temp_df['date']).dt.date
    temp_df["close"] = pd.to_numeric(temp_df["close"], errors="coerce")
    temp_df["position"] = pd.to_numeric(temp_df["position"], errors="coerce")
    return temp_df


if __name__ == "__main__":
    fund_stock_position_lg_df = fund_stock_position_lg()
    print(fund_stock_position_lg_df)

    fund_balance_position_lg_df = fund_balance_position_lg()
    print(fund_balance_position_lg_df)

    fund_linghuo_position_lg_df = fund_linghuo_position_lg()
    print(fund_linghuo_position_lg_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/10/11 17:20
Desc: 同花顺-财务指标-主要指标
https://basic.10jqka.com.cn/new/000063/finance.html
"""

import json

import pandas as pd
import requests
from bs4 import BeautifulSoup


def stock_financial_abstract_ths(
    symbol: str = "000063", indicator: str = "按报告期"
) -> pd.DataFrame:
    """
    同花顺-财务指标-主要指标
    https://basic.10jqka.com.cn/new/000063/finance.html
    :param symbol: 股票代码
    :type symbol: str
    :param indicator: 指标; choice of {"按报告期", "按年度", "按单季度"}
    :type indicator: str
    :return: 同花顺-财务指标-主要指标
    :rtype: pandas.DataFrame
    """
    url = f"https://basic.10jqka.com.cn/new/{symbol}/finance.html"
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) "
        "AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/89.0.4389.90 Safari/537.36",
    }
    r = requests.get(url, headers=headers)
    soup = BeautifulSoup(r.text, features="lxml")
    data_text = soup.find(name="p", attrs={"id": "main"}).string
    data_json = json.loads(data_text)
    df_index = [
        item[0] if isinstance(item, list) else item for item in data_json["title"]
    ]
    if indicator == "按报告期":
        temp_df = pd.DataFrame(
            data_json["report"][1:], columns=data_json["report"][0], index=df_index[1:]
        )
    elif indicator == "按单季度":
        temp_df = pd.DataFrame(
            data_json["simple"][1:], columns=data_json["simple"][0], index=df_index[1:]
        )
    else:
        temp_df = pd.DataFrame(
            data_json["year"][1:], columns=data_json["year"][0], index=df_index[1:]
        )
    temp_df = temp_df.T
    temp_df.reset_index(inplace=True)
    temp_df.rename(columns={"index": "报告期"}, inplace=True)
    return temp_df


def stock_financial_debt_ths(
    symbol: str = "000063", indicator: str = "按报告期"
) -> pd.DataFrame:
    """
    同花顺-财务指标-资产负债表
    https://basic.10jqka.com.cn/new/000063/finance.html
    https://basic.10jqka.com.cn/api/stock/finance/000063_debt.json
    :param symbol: 股票代码
    :type symbol: str
    :param indicator: 指标; choice of {"按报告期", "按年度"}
    :type indicator: str
    :return: 同花顺-财务指标-资产负债表
    :rtype: pandas.DataFrame
    """
    url = f"https://basic.10jqka.com.cn/api/stock/finance/{symbol}_debt.json"
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) "
        "AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/89.0.4389.90 Safari/537.36",
    }
    r = requests.get(url, headers=headers)
    data_json = json.loads(json.loads(r.text)["flashData"])
    df_index = [
        item[0] if isinstance(item, list) else item for item in data_json["title"]
    ]
    if indicator == "按报告期":
        temp_df = pd.DataFrame(
            data_json["report"][1:], columns=data_json["report"][0], index=df_index[1:]
        )
    else:
        temp_df = pd.DataFrame(
            data_json["year"][1:], columns=data_json["year"][0], index=df_index[1:]
        )
    temp_df = temp_df.T
    temp_df.reset_index(inplace=True)
    temp_df.rename(columns={"index": "报告期"}, inplace=True)
    return temp_df


def stock_financial_benefit_ths(
    symbol: str = "000063", indicator: str = "按报告期"
) -> pd.DataFrame:
    """
    同花顺-财务指标-利润表
    https://basic.10jqka.com.cn/new/000063/finance.html
    https://basic.10jqka.com.cn/api/stock/finance/000063_benefit.json
    :param symbol: 股票代码
    :type symbol: str
    :param indicator: 指标; choice of {"按报告期","按单季度", "按年度"}
    :type indicator: str
    :return: 同花顺-财务指标-利润表
    :rtype: pandas.DataFrame
    """
    url = f"https://basic.10jqka.com.cn/api/stock/finance/{symbol}_benefit.json"
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) "
        "AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/89.0.4389.90 Safari/537.36",
    }
    r = requests.get(url, headers=headers)
    data_json = json.loads(json.loads(r.text)["flashData"])
    df_index = [
        item[0] if isinstance(item, list) else item for item in data_json["title"]
    ]
    if indicator == "按报告期":
        temp_df = pd.DataFrame(
            data_json["report"][1:], columns=data_json["report"][0], index=df_index[1:]
        )
    elif indicator == "按单季度":
        temp_df = pd.DataFrame(
            data_json["simple"][1:], columns=data_json["simple"][0], index=df_index[1:]
        )
    else:
        temp_df = pd.DataFrame(
            data_json["year"][1:], columns=data_json["year"][0], index=df_index[1:]
        )
    temp_df = temp_df.T
    temp_df.reset_index(inplace=True)
    temp_df.rename(columns={"index": "报告期"}, inplace=True)
    return temp_df


def stock_financial_cash_ths(
    symbol: str = "000063", indicator: str = "按报告期"
) -> pd.DataFrame:
    """
    同花顺-财务指标-现金流量表
    https://basic.10jqka.com.cn/new/000063/finance.html
    https://basic.10jqka.com.cn/api/stock/finance/000063_cash.json
    :param symbol: 股票代码
    :type symbol: str
    :param indicator: 指标; choice of {"按报告期","按单季度", "按年度"}
    :type indicator: str
    :return: 同花顺-财务指标-现金流量表
    :rtype: pandas.DataFrame
    """
    url = f"https://basic.10jqka.com.cn/api/stock/finance/{symbol}_cash.json"
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) "
        "AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/89.0.4389.90 Safari/537.36",
    }
    r = requests.get(url, headers=headers)
    data_json = json.loads(json.loads(r.text)["flashData"])
    df_index = [
        item[0] if isinstance(item, list) else item for item in data_json["title"]
    ]
    if indicator == "按报告期":
        temp_df = pd.DataFrame(
            data_json["report"][1:], columns=data_json["report"][0], index=df_index[1:]
        )
    elif indicator == "按单季度":
        temp_df = pd.DataFrame(
            data_json["simple"][1:], columns=data_json["simple"][0], index=df_index[1:]
        )
    else:
        temp_df = pd.DataFrame(
            data_json["year"][1:], columns=data_json["year"][0], index=df_index[1:]
        )
    temp_df = temp_df.T
    temp_df.reset_index(inplace=True)
    temp_df.rename(columns={"index": "报告期"}, inplace=True)
    return temp_df


if __name__ == "__main__":
    stock_financial_abstract_ths_df = stock_financial_abstract_ths(
        symbol="000063", indicator="按报告期"
    )
    print(stock_financial_abstract_ths_df)

    stock_financial_abstract_ths_df = stock_financial_abstract_ths(
        symbol="000063", indicator="按年度"
    )
    print(stock_financial_abstract_ths_df)

    stock_financial_abstract_ths_df = stock_financial_abstract_ths(
        symbol="000063", indicator="按单季度"
    )
    print(stock_financial_abstract_ths_df)

    stock_financial_debt_ths_df = stock_financial_debt_ths(
        symbol="002004", indicator="按报告期"
    )
    print(stock_financial_debt_ths_df)

    stock_financial_debt_ths_df = stock_financial_debt_ths(
        symbol="000063", indicator="按年度"
    )
    print(stock_financial_debt_ths_df)

    stock_financial_benefit_ths_df = stock_financial_benefit_ths(
        symbol="000063", indicator="按报告期"
    )
    print(stock_financial_benefit_ths_df)

    stock_financial_benefit_ths_df = stock_financial_benefit_ths(
        symbol="000063", indicator="按年度"
    )
    print(stock_financial_benefit_ths_df)

    stock_financial_benefit_ths_df = stock_financial_benefit_ths(
        symbol="000063", indicator="按报告期"
    )
    print(stock_financial_benefit_ths_df)

    stock_financial_cash_ths_df = stock_financial_cash_ths(
        symbol="000063", indicator="按年度"
    )
    print(stock_financial_cash_ths_df)

    stock_financial_cash_ths_df = stock_financial_cash_ths(
        symbol="000063", indicator="按年度"
    )
    print(stock_financial_cash_ths_df)

    stock_financial_cash_ths_df = stock_financial_cash_ths(
        symbol="000063", indicator="按单季度"
    )
    print(stock_financial_cash_ths_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2023/11/10 15:20
Desc: 新浪财经-中行人民币牌价历史数据查询
https://biz.finance.sina.com.cn/forex/forex.php?startdate=2012-01-01&enddate=2021-06-14&money_code=EUR&type=0
"""
from functools import lru_cache
from io import StringIO

import pandas as pd
import requests
from bs4 import BeautifulSoup
from tqdm import tqdm


@lru_cache()
def _currency_boc_sina_map(
        start_date: str = "20210614", end_date: str = "20230810"
) -> dict:
    """
    外汇 symbol 和代码映射
    https://biz.finance.sina.com.cn/forex/forex.php?startdate=2012-01-01&enddate=2021-06-14&money_code=EUR&type=0
    :param start_date: 开始交易日
    :type start_date: str
    :param end_date: 结束交易日
    :type end_date: str
    :return: 外汇 symbol 和代码映射
    :rtype: dict
    """
    url = "http://biz.finance.sina.com.cn/forex/forex.php"
    params = {
        "startdate": "-".join([start_date[:4], start_date[4:6], start_date[6:]]),
        "enddate": "-".join([end_date[:4], end_date[4:6], end_date[6:]]),
        "money_code": "EUR",
        "type": "0",
    }
    r = requests.get(url, params=params)
    r.encoding = "gbk"
    soup = BeautifulSoup(r.text, "lxml")
    data_dict = dict(
        zip(
            [
                item.text
                for item in soup.find(attrs={"id": "money_code"}).find_all("option")
            ],
            [
                item["value"]
                for item in soup.find(attrs={"id": "money_code"}).find_all("option")
            ],
        )
    )
    return data_dict


def currency_boc_sina(
        symbol: str = "美元", start_date: str = "20230304", end_date: str = "20231110"
) -> pd.DataFrame:
    """
    新浪财经-中行人民币牌价历史数据查询
    https://biz.finance.sina.com.cn/forex/forex.php?startdate=2012-01-01&enddate=2021-06-14&money_code=EUR&type=0
    :param symbol: choice of {'美元', '英镑', '欧元', '澳门元', '泰国铢', '菲律宾比索', '港币', '瑞士法郎', '新加坡元', '瑞典克朗', '丹麦克朗', '挪威克朗', '日元', '加拿大元', '澳大利亚元', '新西兰元', '韩国元'}
    :type symbol: str
    :param start_date: 开始交易日
    :type start_date: str
    :param end_date: 结束交易日
    :type end_date: str
    :return: 中行人民币牌价历史数据查询
    :rtype: pandas.DataFrame
    """
    data_dict = _currency_boc_sina_map(start_date=start_date, end_date=end_date)
    url = "http://biz.finance.sina.com.cn/forex/forex.php"
    params = {
        "money_code": data_dict[symbol],
        "type": "0",
        "startdate": "-".join([start_date[:4], start_date[4:6], start_date[6:]]),
        "enddate": "-".join([end_date[:4], end_date[4:6], end_date[6:]]),
        "page": "1",
        "call_type": "ajax",
    }
    r = requests.get(url, params=params)
    soup = BeautifulSoup(r.text, "lxml")
    soup.find(attrs={"id": "money_code"})
    page_element_list = soup.find_all("a", attrs={"class": "page"})
    page_num = int(page_element_list[-2].text) if len(page_element_list) != 0 else 1
    big_df = pd.DataFrame()
    for page in tqdm(range(1, page_num + 1), leave=False):
        params.update({"page": page})
        r = requests.get(url, params=params)
        temp_df = pd.read_html(StringIO(r.text), header=0)[0]
        big_df = pd.concat([big_df, temp_df], ignore_index=True)
    big_df.columns = [
        "日期",
        "中行汇买价",
        "中行钞买价",
        "中行钞卖价/汇卖价",
        "央行中间价",
    ]
    big_df["日期"] = pd.to_datetime(big_df["日期"], errors="coerce").dt.date
    big_df["中行汇买价"] = pd.to_numeric(big_df["中行汇买价"], errors="coerce")
    big_df["中行钞买价"] = pd.to_numeric(big_df["中行钞买价"], errors="coerce")
    big_df["中行钞卖价/汇卖价"] = pd.to_numeric(big_df["中行钞卖价/汇卖价"], errors="coerce")
    big_df["央行中间价"] = pd.to_numeric(big_df["央行中间价"], errors="coerce")
    big_df.sort_values(by=["日期"], inplace=True, ignore_index=True)
    return big_df


if __name__ == "__main__":
    currency_boc_sina_df = currency_boc_sina(
        symbol="美元", start_date="20230304", end_date="20231110"
    )
    print(currency_boc_sina_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/4/11 17:40
Desc: 申万宏源研究-申万指数-指数发布-基金指数-实时行情
https://www.swsresearch.com/institute_sw/allIndex/releasedIndex
"""

import pandas as pd
import requests

from byteapp_ak.utils.cons import headers


def index_realtime_fund_sw(symbol: str = "基础一级") -> pd.DataFrame:
    """
    申万宏源研究-申万指数-指数发布-基金指数-实时行情
    https://www.swsresearch.com/institute_sw/allIndex/releasedIndex
    :param symbol: choice of {"基础一级", "基础二级", "基础三级", "特色指数"}
    :type symbol: str
    :return: 基金指数-实时行情
    :rtype: pandas.DataFrame
    """
    url = "https://www.swsresearch.com/insWechatSw/fundIndex/pageList"
    payload = {
        "pageNo": 1,
        "pageSize": 50,
        "indexTypeName": symbol,
        "sortField": "",
        "rule": "",
        "indexType": 1,
    }
    r = requests.post(url, json=payload, headers=headers)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["data"]["list"])
    temp_df.rename(
        columns={
            "swIndexCode": "指数代码",
            "swIndexName": "指数名称",
            "lastCloseIndex": "昨收盘",
            "lastMarkup": "日涨跌幅",
            "yearMarkup": "年涨跌幅",
        },
        inplace=True,
    )
    temp_df = temp_df[
        [
            "指数代码",
            "指数名称",
            "昨收盘",
            "日涨跌幅",
            "年涨跌幅",
        ]
    ]
    temp_df["昨收盘"] = pd.to_numeric(temp_df["昨收盘"], errors="coerce")
    temp_df["日涨跌幅"] = pd.to_numeric(temp_df["日涨跌幅"], errors="coerce")
    temp_df["年涨跌幅"] = pd.to_numeric(temp_df["年涨跌幅"], errors="coerce")
    return temp_df


def index_hist_fund_sw(symbol: str = "807200", period: str = "day") -> pd.DataFrame:
    """
    申万宏源研究-申万指数-指数发布-基金指数-历史行情
    https://www.swsresearch.com/institute_sw/allIndex/releasedIndex/fundDetail?code=807100
    :param symbol: 基金指数代码
    :type symbol: str
    :param period: 周期
    :type period: str
    :return: 历史行情
    :rtype: pandas.DataFrame
    """
    period_map = {
        "day": "DAY",
        "week": "WEEK",
        "month": "MONTH",
    }
    url = "https://www.swsresearch.com/insWechatSw/fundIndex/getFundKChartData"
    payload = {"swIndexCode": symbol, "type": period_map[period]}
    r = requests.post(url, json=payload, headers=headers)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["data"])
    temp_df.rename(
        columns={
            "bargaindate": "日期",
            "swIndexName": "指数名称",
            "swindexcode": "指数代码",
            "closeindex": "收盘指数",
            "maxindex": "最高指数",
            "minindex": "最低指数",
            "openindex": "开盘指数",
            "markup": "涨跌幅",
        },
        inplace=True,
    )
    temp_df = temp_df[
        [
            "日期",
            "收盘指数",
            "开盘指数",
            "最高指数",
            "最低指数",
            "涨跌幅",
        ]
    ]
    temp_df["日期"] = pd.to_datetime(temp_df["日期"], errors="coerce").dt.date
    temp_df["收盘指数"] = pd.to_numeric(temp_df["收盘指数"], errors="coerce")
    temp_df["最高指数"] = pd.to_numeric(temp_df["最高指数"], errors="coerce")
    temp_df["最低指数"] = pd.to_numeric(temp_df["最低指数"], errors="coerce")
    temp_df["开盘指数"] = pd.to_numeric(temp_df["开盘指数"], errors="coerce")
    temp_df["涨跌幅"] = pd.to_numeric(temp_df["涨跌幅"], errors="coerce")
    return temp_df


if __name__ == "__main__":
    index_realtime_fund_sw_df = index_realtime_fund_sw(symbol="基础一级")
    print(index_realtime_fund_sw_df)

    index_realtime_fund_sw_df = index_realtime_fund_sw(symbol="基础二级")
    print(index_realtime_fund_sw_df)

    index_realtime_fund_sw_df = index_realtime_fund_sw(symbol="基础三级")
    print(index_realtime_fund_sw_df)

    index_realtime_fund_sw_df = index_realtime_fund_sw(symbol="特色指数")
    print(index_realtime_fund_sw_df)

    index_hist_fund_sw_df = index_hist_fund_sw(symbol="807200", period="day")
    print(index_hist_fund_sw_df)

    index_hist_fund_sw_df = index_hist_fund_sw(symbol="807200", period="week")
    print(index_hist_fund_sw_df)

    index_hist_fund_sw_df = index_hist_fund_sw(symbol="807200", period="month")
    print(index_hist_fund_sw_df)



# -*- coding:utf-8 -*-
# !/usr/bin/env python
"""
Date: 2022/8/29 14:20
Desc: 东方财富网-数据中心-股票回购-股票回购数据
https://data.eastmoney.com/gphg/hglist.html
"""
import pandas as pd
import requests
from tqdm import tqdm


def stock_repurchase_em() -> pd.DataFrame:
    """
    东方财富网-数据中心-股票回购-股票回购数据
    https://data.eastmoney.com/gphg/hglist.html
    :return: 股票回购数据
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "sortColumns": "UPD,DIM_DATE,DIM_SCODE",
        "sortTypes": "-1,-1,-1",
        "pageSize": "500",
        "pageNumber": "1",
        "reportName": "RPTA_WEB_GETHGLIST_NEW",
        "columns": "ALL",
        "source": "WEB",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    total_page = data_json["result"]["pages"]
    big_df = pd.DataFrame()
    for page in tqdm(range(1, int(total_page) + 1), leave=False):
        params.update({"pageNumber": page})
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["result"]["data"])
        big_df = pd.concat([big_df, temp_df], ignore_index=True)
    big_df.rename(
        {
            "DIM_SCODE": "股票代码",
            "SECURITYSHORTNAME": "股票简称",
            "NEWPRICE": "最新价",
            "REPURPRICECAP": "计划回购价格区间",
            "REPURNUMLOWER": "计划回购数量区间-下限",
            "REPURNUMCAP": "计划回购数量区间-上限",
            "ZSZXX": "占公告前一日总股本比例-下限",
            "ZSZSX": "占公告前一日总股本比例-上限",
            "JEXX": "计划回购金额区间-下限",
            "JESX": "计划回购金额区间-上限",
            "DIM_TRADEDATE": "回购起始时间",
            "REPURPROGRESS": "实施进度",
            "REPURPRICELOWER1": "已回购股份价格区间-下限",
            "REPURPRICECAP1": "已回购股份价格区间-上限",
            "REPURNUM": "已回购股份数量",
            "REPURAMOUNT": "已回购金额",
            "UPDATEDATE": "最新公告日期",
        },
        axis="columns",
        inplace=True,
    )
    big_df = big_df[
        [
            "股票代码",
            "股票简称",
            "最新价",
            "计划回购价格区间",
            "计划回购数量区间-下限",
            "计划回购数量区间-上限",
            "占公告前一日总股本比例-下限",
            "占公告前一日总股本比例-上限",
            "计划回购金额区间-下限",
            "计划回购金额区间-上限",
            "回购起始时间",
            "实施进度",
            "已回购股份价格区间-下限",
            "已回购股份价格区间-上限",
            "已回购股份数量",
            "已回购金额",
            "最新公告日期",
        ]
    ]
    big_df.reset_index(inplace=True)
    big_df.rename(
        {
            "index": "序号",
        },
        axis="columns",
        inplace=True,
    )
    big_df["序号"] = big_df.index + 1
    process_map = {
        "001": "董事会预案",
        "002": "股东大会通过",
        "003": "股东大会否决",
        "004": "实施中",
        "005": "停止实施",
        "006": "完成实施",
    }
    big_df["实施进度"] = big_df["实施进度"].map(process_map)
    big_df["回购起始时间"] = pd.to_datetime(big_df["回购起始时间"]).dt.date
    big_df["最新公告日期"] = pd.to_datetime(big_df["最新公告日期"]).dt.date
    big_df["最新价"] = pd.to_numeric(big_df["最新价"])
    big_df["计划回购价格区间"] = pd.to_numeric(big_df["计划回购价格区间"])
    big_df["计划回购数量区间-下限"] = pd.to_numeric(big_df["计划回购数量区间-下限"])
    big_df["计划回购数量区间-上限"] = pd.to_numeric(big_df["计划回购数量区间-上限"])
    big_df["占公告前一日总股本比例-上限"] = pd.to_numeric(big_df["占公告前一日总股本比例-上限"])
    big_df["占公告前一日总股本比例-下限"] = pd.to_numeric(big_df["占公告前一日总股本比例-下限"])
    big_df["计划回购金额区间-上限"] = pd.to_numeric(big_df["计划回购金额区间-上限"])
    big_df["计划回购金额区间-下限"] = pd.to_numeric(big_df["计划回购金额区间-下限"])
    big_df["已回购股份价格区间-下限"] = pd.to_numeric(big_df["已回购股份价格区间-下限"])
    big_df["已回购股份价格区间-上限"] = pd.to_numeric(big_df["已回购股份价格区间-上限"])
    big_df["已回购股份数量"] = pd.to_numeric(big_df["已回购股份数量"])
    big_df["已回购金额"] = pd.to_numeric(big_df["已回购金额"])
    return big_df


if __name__ == "__main__":
    stock_repurchase_em_df = stock_repurchase_em()
    print(stock_repurchase_em_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/1/18 17:40
Desc: 东方财富网-行情中心-期货市场-国际期货
https://quote.eastmoney.com/center/gridlist.html#futures_global
"""
import math

import pandas as pd
import requests

from byteapp_ak.utils.tqdm import get_tqdm


def futures_global_em():
    """
    东方财富网-行情中心-期货市场-国际期货
    https://quote.eastmoney.com/center/gridlist.html#futures_global
    :return: 行情数据
    :rtype: pandas.DataFrame
    """
    url = "https://futsseapi.eastmoney.com/list/COMEX,NYMEX,COBOT,SGX,NYBOT,LME,MDEX,TOCOM,IPE"
    params = {
        'orderBy': 'zdf',
        'sort': 'desc',
        'pageSize': '20',
        'pageIndex': '0',
        'token': '58b2fa8f54638b60b87d69b31969089c',
        'field': 'dm,sc,name,p,zsjd,zde,zdf,f152,o,h,l,zjsj,vol,wp,np,ccl',
        'blockName': 'callback',
        '_': '1705570814466'
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    total_num = data_json['total']
    total_page = math.ceil(total_num / 20) - 1
    tqdm = get_tqdm()
    big_df = pd.DataFrame()
    for page in tqdm(range(total_page), leave=False):
        params.update({'pageIndex': page})
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json['list'])
        big_df = pd.concat([big_df, temp_df], ignore_index=True)
    big_df.reset_index(inplace=True)
    big_df['index'] = big_df['index'] + 1
    big_df.rename(columns={
        'index': "序号",
        'np': "卖盘",
        'h': "最新价",
        'dm': "代码",
        'zsjd': "-",
        'l': "最低",
        'ccl': "持仓量",
        'o': "今开",
        'p': "最高",
        'sc': "-",
        'vol': "成交量",
        'name': "名称",
        'wp': "买盘",
        'zde': "涨跌额",
        'zdf': "涨跌幅",
        'zjsj': "昨结"
    }, inplace=True)
    big_df = big_df[[
        "序号",
        "代码",
        "名称",
        "最新价",
        "涨跌额",
        "涨跌幅",
        "今开",
        "最高",
        "最低",
        "昨结",
        "成交量",
        "买盘",
        "卖盘",
        "持仓量",
    ]]
    big_df['最新价'] = pd.to_numeric(big_df['最新价'], errors="coerce")
    big_df['涨跌额'] = pd.to_numeric(big_df['涨跌额'], errors="coerce")
    big_df['涨跌幅'] = pd.to_numeric(big_df['涨跌幅'], errors="coerce")
    big_df['今开'] = pd.to_numeric(big_df['今开'], errors="coerce")
    big_df['最高'] = pd.to_numeric(big_df['最高'], errors="coerce")
    big_df['最低'] = pd.to_numeric(big_df['最低'], errors="coerce")
    big_df['昨结'] = pd.to_numeric(big_df['昨结'], errors="coerce")
    big_df['成交量'] = pd.to_numeric(big_df['成交量'], errors="coerce")
    big_df['买盘'] = pd.to_numeric(big_df['买盘'], errors="coerce")
    big_df['卖盘'] = pd.to_numeric(big_df['卖盘'], errors="coerce")
    big_df['持仓量'] = pd.to_numeric(big_df['持仓量'], errors="coerce")
    return big_df


if __name__ == '__main__':
    futures_global_em_df = futures_global_em()
    print(futures_global_em_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/1/6 15:00
Desc: 东方财富网-数据中心-研究报告-盈利预测
https://data.eastmoney.com/report/profitforecast.jshtml
"""

import pandas as pd
import requests

from byteapp_ak.utils.tqdm import get_tqdm


def stock_profit_forecast_em(symbol: str = "") -> pd.DataFrame:
    """
    东方财富网-数据中心-研究报告-盈利预测
    https://data.eastmoney.com/report/profitforecast.jshtml
    :param symbol: "", 默认为获取全部数据; symbol="船舶制造", 则获取具体行业板块的数据;
    行业板块可以通过 ak.stock_board_industry_name_em() 接口获取
    :type symbol: str
    :return: 盈利预测
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "reportName": "RPT_WEB_RESPREDICT",
        "columns": "WEB_RESPREDICT",
        "pageNumber": "1",
        "pageSize": "500",
        "sortTypes": "-1",
        "sortColumns": "RATING_ORG_NUM",
        "p": "1",
        "pageNo": "1",
        "pageNum": "1",
    }
    if symbol:
        params.update({"filter": f'(INDUSTRY_BOARD="{symbol}")'})
    r = requests.get(url, params=params)
    data_json = r.json()
    page_num = int(data_json["result"]["pages"])
    big_df = pd.DataFrame()
    tqdm = get_tqdm()
    for page in tqdm(range(1, page_num + 1), leave=False):
        params.update(
            {
                "pageNumber": page,
                "p": page,
                "pageNo": page,
                "pageNum": page,
            }
        )
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["result"]["data"])
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)

    big_df.reset_index(inplace=True)
    big_df["index"] = big_df.index + 1
    year1 = str(big_df["YEAR1"].mode().values[0])
    year2 = str(big_df["YEAR2"].mode().values[0])
    year3 = str(big_df["YEAR3"].mode().values[0])
    year4 = str(big_df["YEAR4"].mode().values[0])
    big_df.columns = [
        "序号",
        "-",
        "代码",
        "名称",
        "研报数",
        "机构投资评级(近六个月)-买入",
        "机构投资评级(近六个月)-增持",
        "机构投资评级(近六个月)-中性",
        "机构投资评级(近六个月)-减持",
        "机构投资评级(近六个月)-卖出",
        "-",
        "_",
        f"{year1}预测每股收益",
        "-",
        "_",
        f"{year2}预测每股收益",
        "-",
        "_",
        f"{year3}预测每股收益",
        "-",
        "_",
        f"{year4}预测每股收益",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
    ]
    big_df = big_df[
        [
            "序号",
            "代码",
            "名称",
            "研报数",
            "机构投资评级(近六个月)-买入",
            "机构投资评级(近六个月)-增持",
            "机构投资评级(近六个月)-中性",
            "机构投资评级(近六个月)-减持",
            "机构投资评级(近六个月)-卖出",
            f"{year1}预测每股收益",
            f"{year2}预测每股收益",
            f"{year3}预测每股收益",
            f"{year4}预测每股收益",
        ]
    ]
    big_df["机构投资评级(近六个月)-买入"] = big_df[
        "机构投资评级(近六个月)-买入"
    ].fillna(0)
    big_df["机构投资评级(近六个月)-增持"] = big_df[
        "机构投资评级(近六个月)-增持"
    ].fillna(0)
    big_df["研报数"] = pd.to_numeric(big_df["研报数"], errors="coerce")
    big_df["机构投资评级(近六个月)-买入"] = pd.to_numeric(
        big_df["机构投资评级(近六个月)-买入"], errors="coerce"
    )
    big_df["机构投资评级(近六个月)-增持"] = pd.to_numeric(
        big_df["机构投资评级(近六个月)-增持"], errors="coerce"
    )
    big_df["机构投资评级(近六个月)-中性"] = pd.to_numeric(
        big_df["机构投资评级(近六个月)-中性"], errors="coerce"
    )
    big_df["机构投资评级(近六个月)-减持"] = pd.to_numeric(
        big_df["机构投资评级(近六个月)-减持"], errors="coerce"
    )
    big_df["机构投资评级(近六个月)-卖出"] = pd.to_numeric(
        big_df["机构投资评级(近六个月)-卖出"], errors="coerce"
    )
    big_df["机构投资评级(近六个月)-中性"] = big_df[
        "机构投资评级(近六个月)-中性"
    ].fillna(0)
    big_df["机构投资评级(近六个月)-减持"] = big_df[
        "机构投资评级(近六个月)-减持"
    ].fillna(0)
    big_df["机构投资评级(近六个月)-卖出"] = big_df[
        "机构投资评级(近六个月)-卖出"
    ].fillna(0)
    big_df[f"{year1}预测每股收益"] = pd.to_numeric(
        big_df[f"{year1}预测每股收益"], errors="coerce"
    )
    big_df[f"{year2}预测每股收益"] = pd.to_numeric(
        big_df[f"{year2}预测每股收益"], errors="coerce"
    )
    big_df[f"{year3}预测每股收益"] = pd.to_numeric(
        big_df[f"{year3}预测每股收益"], errors="coerce"
    )
    big_df[f"{year4}预测每股收益"] = pd.to_numeric(
        big_df[f"{year4}预测每股收益"], errors="coerce"
    )
    big_df.sort_values(["研报数"], ascending=False, inplace=True, ignore_index=True)
    big_df["序号"] = range(1, len(big_df) + 1)
    return big_df


if __name__ == "__main__":
    stock_profit_forecast_em_df = stock_profit_forecast_em(symbol="航运港口")
    print(stock_profit_forecast_em_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2021/1/10 13:58
Desc: 得到 99 期货网的原始数据
"""
import requests
import pickle
from bs4 import BeautifulSoup
import time


url = "http://service.99qh.com/Storage/Storage.aspx?page=99qh"

sample_headers = {
    "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3",
    "Host": "service.99qh.com",
    "Origin": "http://service.99qh.com",
    "Referer": "http://www.99qh.com/d/store.aspx",
    "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/77.0.3865.90 Safari/537.36"
}

res = requests.get(url, headers=sample_headers)

soup = BeautifulSoup(res.text, "lxml")
view_state = soup.find_all(attrs={"id": "__VIEWSTATE"})[0]["value"]
even_validation = soup.find_all(attrs={"id": "__EVENTVALIDATION"})[0]["value"]

qh_headers = {
    "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3",
    "Accept-Encoding": "gzip, deflate",
    "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
    "Cache-Control": "max-age=0",
    "Connection": "keep-alive",
    "Content-Length": "4458",
    "Content-Type": "application/x-www-form-urlencoded",
    "Host": "service.99qh.com",
    "Origin": "http://service.99qh.com",
    "Referer": "http://service.99qh.com/Storage/Storage.aspx?page=99qh",
    "Upgrade-Insecure-Requests": "1",
    "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/77.0.3865.90 Safari/537.36"
}

code_temp_list = [item["value"] for item in soup.find_all("select")[0].find_all("option")]
name_temp_list = [item.get_text() for item in soup.find_all("select")[0].find_all("option")]
code_exchange_name_dict = dict(zip(code_temp_list, name_temp_list))
exchange_value = soup.find_all("select")[0].find_all(attrs={"selected": "selected"})[0]["value"]
symbol_list = soup.find_all("select")[1].get_text().split("\n")[1:-1]
symbol_value_list = [item["value"] for item in soup.find_all("select")[1].find_all("option")]
code_symbol_code_dict = dict()
code_symbol_code_dict[exchange_value] = symbol_value_list

exchange_symbol_list = {}
exchange_symbol_value_list = {}
for i in code_temp_list:
    # i = 14
    print(i)
    j = 0
    while j < 5:
        try:
            payload = {
                "__EVENTTARGET": "ddlExchName",
                "__EVENTARGUMENT": "",
                "__LASTFOCUS": "",
                "__VIEWSTATE": view_state,
                "__VIEWSTATEGENERATOR": "6EAC22FA",
                "__EVENTVALIDATION": even_validation,
                "ddlExchName": i,
                # "ddlGoodsName": 6
            }

            res = requests.post(url, data=payload, headers=qh_headers)
            soup = BeautifulSoup(res.text, "lxml")
            exchange = soup.find_all("select")[0].find_all(attrs={"selected": "selected"})[0].get_text()
            print(exchange)
            exchange_value = soup.find_all("select")[0].find_all(attrs={"selected": "selected"})[0]["value"]
            print(exchange_value)
            symbol_list = soup.find_all("select")[1].get_text().split("\n")[1:-1]
            exchange_symbol_list.update({exchange_value: symbol_list})
            symbol_value_list = [item["value"] for item in soup.find_all("select")[1].find_all("option")]
            exchange_symbol_value_list.update({exchange_value: symbol_value_list})
            view_state = soup.find_all(attrs={"id": "__VIEWSTATE"})[0]["value"]
            even_validation = soup.find_all(attrs={"id": "__EVENTVALIDATION"})[0]["value"]
            time.sleep(5)
        except:
            j += 1
            continue


with open('./byteapp_ak/futures/exchange_symbol_list.pk', 'wb') as f:
    pickle.dump(exchange_symbol_list, f)

with open('./byteapp_ak/futures/exchange_symbol_value_list.pk', 'wb') as f:
    pickle.dump(exchange_symbol_value_list, f)

with open('./byteapp_ak/futures/code_exchange_name_dict.pk', 'wb') as f:
    pickle.dump(code_exchange_name_dict, f)






#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2019/11/14 20:32
Desc: 学术板块配置文件
"""
# EPU
epu_home_url = "http://www.policyuncertainty.com/index.html"

# FF-Factor
ff_home_url = "http://mba.tuck.dartmouth.edu/pages/faculty/ken.french/data_library.html"



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2022/3/5 12:55
Desc: 上登债券信息网-债券成交概览
http://bond.sse.com.cn/data/statistics/overview/turnover/
"""
from io import BytesIO

import pandas as pd
import requests


def bond_cash_summary_sse(date: str = '20210111') -> pd.DataFrame:
    """
    上登债券信息网-市场数据-市场统计-市场概览-债券现券市场概览
    http://bond.sse.com.cn/data/statistics/overview/bondow/
    :param date: 指定日期
    :type date: str
    :return: 债券成交概览
    :rtype: pandas.DataFrame
    """
    url = 'http://query.sse.com.cn/commonExcelDd.do'
    headers = {
        'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9',
        'Referer': 'http://bond.sse.com.cn/',
        'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/87.0.4280.88 Safari/537.36'
    }
    params = {
        'sqlId': 'COMMON_SSEBOND_SCSJ_SCTJ_SCGL_ZQXQSCGL_CX_L',
        'TRADE_DATE': f'{date[:4]}-{date[4:6]}-{date[6:]}',
    }
    r = requests.get(url, params=params, headers=headers)
    temp_df = pd.read_excel(BytesIO(r.content), engine="xlrd")
    temp_df.columns = [
        '债券现货',
        '托管只数',
        '托管市值',
        '托管面值',
        '数据日期',
    ]
    temp_df['托管只数'] = pd.to_numeric(temp_df['托管只数'])
    temp_df['托管市值'] = pd.to_numeric(temp_df['托管市值'])
    temp_df['托管面值'] = pd.to_numeric(temp_df['托管面值'])
    temp_df['数据日期'] = pd.to_datetime(temp_df['数据日期']).dt.date
    return temp_df


def bond_deal_summary_sse(date: str = '20210104') -> pd.DataFrame:
    """
    上登债券信息网-市场数据-市场统计-市场概览-债券成交概览
    http://bond.sse.com.cn/data/statistics/overview/turnover/
    :param date: 指定日期
    :type date: str
    :return: 债券成交概览
    :rtype: pandas.DataFrame
    """
    url = 'http://query.sse.com.cn/commonExcelDd.do'
    headers = {
        'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9',
        'Referer': 'http://bond.sse.com.cn/',
        'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/87.0.4280.88 Safari/537.36'
    }
    params = {
        'sqlId': 'COMMON_SSEBOND_SCSJ_SCTJ_SCGL_ZQCJGL_CX_L',
        'TRADE_DATE': f'{date[:4]}-{date[4:6]}-{date[6:]}',
    }
    r = requests.get(url, params=params, headers=headers)
    temp_df = pd.read_excel(BytesIO(r.content))
    temp_df.columns = [
        '债券类型',
        '当日成交笔数',
        '当日成交金额',
        '当年成交笔数',
        '当年成交金额',
        '数据日期',
    ]
    temp_df['当日成交笔数'] = pd.to_numeric(temp_df['当日成交笔数'])
    temp_df['当日成交金额'] = pd.to_numeric(temp_df['当日成交金额'])
    temp_df['当年成交笔数'] = pd.to_numeric(temp_df['当年成交笔数'])
    temp_df['当年成交金额'] = pd.to_numeric(temp_df['当年成交金额'])
    temp_df['数据日期'] = pd.to_datetime(temp_df['数据日期']).dt.date
    return temp_df


if __name__ == '__main__':
    bond_cash_summary_sse_df = bond_cash_summary_sse(date='20210111')
    print(bond_cash_summary_sse_df)

    bond_summary_sse_df = bond_deal_summary_sse(date='20210111')
    print(bond_summary_sse_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2023/9/2 19:00
Desc: 新浪财经-港股-实时行情数据和历史行情数据(包含前复权和后复权因子)
http://stock.finance.sina.com.cn/hkstock/quotes/00700.html
"""

import pandas as pd
import requests
import py_mini_racer

from byteapp_ak.stock.cons import (
    hk_js_decode,
    hk_sina_stock_dict_payload,
    hk_sina_stock_list_url,
    hk_sina_stock_hist_url,
    hk_sina_stock_hist_hfq_url,
    hk_sina_stock_hist_qfq_url,
)
from byteapp_ak.utils import demjson


def stock_hk_spot() -> pd.DataFrame:
    """
    新浪财经-港股的所有港股的实时行情数据
    http://vip.stock.finance.sina.com.cn/mkt/#qbgg_hk
    :return: 实时行情数据
    :rtype: pandas.DataFrame
    """
    res = requests.get(hk_sina_stock_list_url, params=hk_sina_stock_dict_payload)
    data_json = [
        demjson.decode(tt)
        for tt in [
            item + "}" for item in res.text[1:-1].split("},") if not item.endswith("}")
        ]
    ]
    data_df = pd.DataFrame(data_json)
    data_df = data_df[
        [
            "symbol",
            "name",
            "engname",
            "tradetype",
            "lasttrade",
            "prevclose",
            "open",
            "high",
            "low",
            "volume",
            "amount",
            "ticktime",
            "buy",
            "sell",
            "pricechange",
            "changepercent",
        ]
    ]
    return data_df


def stock_hk_daily(symbol: str = "00981", adjust: str = "") -> pd.DataFrame:
    """
    新浪财经-港股-个股的历史行情数据
    https://stock.finance.sina.com.cn/hkstock/quotes/02912.html
    :param symbol: 可以使用 stock_hk_spot 获取
    :type symbol: str
    :param adjust: "": 返回未复权的数据 ; qfq: 返回前复权后的数据; qfq-factor: 返回前复权因子和调整;
    :type adjust: str
    :return: 指定 adjust 的数据
    :rtype: pandas.DataFrame
    """
    r = requests.get(hk_sina_stock_hist_url.format(symbol))
    js_code = py_mini_racer.MiniRacer()
    js_code.eval(hk_js_decode)
    dict_list = js_code.call(
        "d", r.text.split("=")[1].split(";")[0].replace('"', "")
    )  # 执行js解密代码
    data_df = pd.DataFrame(dict_list)
    data_df.index = pd.to_datetime(data_df["date"]).dt.date
    del data_df["date"]
    data_df = data_df.astype("float")

    if adjust == "":
        data_df.reset_index(inplace=True)
        data_df["date"] = pd.to_datetime(data_df["date"]).dt.date
        return data_df

    if adjust == "hfq":
        r = requests.get(hk_sina_stock_hist_hfq_url.format(symbol))
        try:
            hfq_factor_df = pd.DataFrame(
                eval(r.text.split("=")[1].split("\n")[0])["data"]
            )
            if len(hfq_factor_df) == 1:
                data_df.reset_index(inplace=True)
                data_df["date"] = pd.to_datetime(data_df["date"]).dt.date
                return data_df
        except SyntaxError:
            data_df.reset_index(inplace=True)
            data_df["date"] = pd.to_datetime(data_df["date"]).dt.date
            return data_df
        hfq_factor_df.columns = ["date", "hfq_factor", "cash"]
        hfq_factor_df.index = pd.to_datetime(hfq_factor_df.date)
        del hfq_factor_df["date"]

        # 处理复权因子
        temp_date_range = pd.date_range(
            "1900-01-01", hfq_factor_df.index[0].isoformat()
        )
        temp_df = pd.DataFrame(range(len(temp_date_range)), temp_date_range)
        new_range = pd.merge(
            temp_df, hfq_factor_df, left_index=True, right_index=True, how="outer"
        )
        try:
            # try for pandas >= 2.1.0
            new_range.ffill(inplace=True)
        except Exception:
            try:
                new_range.fillna(method="ffill", inplace=True)
            except Exception as e:
                print("Error:", e)
        new_range = new_range.iloc[:, [1, 2]]

        temp_df = pd.merge(
            data_df, new_range, left_index=True, right_index=True, how="outer"
        )
        try:
            # try for pandas >= 2.1.0
            temp_df.ffill(inplace=True)
        except Exception:
            try:
                # try for pandas < 2.1.0
                temp_df.fillna(method="ffill", inplace=True)
            except Exception as e:
                print("Error:", e)
        temp_df.drop_duplicates(
            subset=["open", "high", "low", "close", "volume"], inplace=True
        )
        temp_df = temp_df.astype(float)
        temp_df["open"] = temp_df["open"] * temp_df["hfq_factor"] + temp_df["cash"]
        temp_df["high"] = temp_df["high"] * temp_df["hfq_factor"] + temp_df["cash"]
        temp_df["close"] = temp_df["close"] * temp_df["hfq_factor"] + temp_df["cash"]
        temp_df["low"] = temp_df["low"] * temp_df["hfq_factor"] + temp_df["cash"]
        temp_df = temp_df.apply(lambda x: round(x, 4))
        temp_df.dropna(how="any", inplace=True)
        temp_df = temp_df.iloc[:, :-2]
        temp_df.reset_index(inplace=True)
        temp_df.rename({"index": "date"}, axis="columns", inplace=True)
        temp_df["date"] = pd.to_datetime(temp_df["date"]).dt.date
        return temp_df

    if adjust == "qfq":
        r = requests.get(hk_sina_stock_hist_qfq_url.format(symbol))
        try:
            qfq_factor_df = pd.DataFrame(
                eval(r.text.split("=")[1].split("\n")[0])["data"]
            )
            if len(qfq_factor_df) == 1:
                data_df.reset_index(inplace=True)
                data_df["date"] = pd.to_datetime(data_df["date"]).dt.date
                return data_df

        except SyntaxError:
            data_df.reset_index(inplace=True)
            data_df["date"] = pd.to_datetime(data_df["date"]).dt.date
            return data_df
        qfq_factor_df.columns = ["date", "qfq_factor"]
        qfq_factor_df.index = pd.to_datetime(qfq_factor_df.date)
        del qfq_factor_df["date"]

        temp_date_range = pd.date_range(
            "1900-01-01", qfq_factor_df.index[0].isoformat()
        )
        temp_df = pd.DataFrame(range(len(temp_date_range)), temp_date_range)
        new_range = pd.merge(
            temp_df, qfq_factor_df, left_index=True, right_index=True, how="outer"
        )
        try:
            # try for pandas >= 2.1.0
            new_range.ffill(inplace=True)
        except Exception:
            try:
                # try for pandas < 2.1.0
                new_range.fillna(method="ffill", inplace=True)
            except Exception as e:
                print("Error:", e)
        new_range = new_range.iloc[:, [1]]

        temp_df = pd.merge(
            data_df, new_range, left_index=True, right_index=True, how="outer"
        )
        try:
            # try for pandas >= 2.1.0
            temp_df.ffill(inplace=True)
        except Exception:
            try:
                # try for pandas < 2.1.0
                temp_df.fillna(method="ffill", inplace=True)
            except Exception as e:
                print("Error:", e)
        temp_df.drop_duplicates(
            subset=["open", "high", "low", "close", "volume"], inplace=True
        )
        temp_df = temp_df.astype(float)
        temp_df["open"] = temp_df["open"] * temp_df["qfq_factor"]
        temp_df["high"] = temp_df["high"] * temp_df["qfq_factor"]
        temp_df["close"] = temp_df["close"] * temp_df["qfq_factor"]
        temp_df["low"] = temp_df["low"] * temp_df["qfq_factor"]
        temp_df = temp_df.apply(lambda x: round(x, 4))
        temp_df.dropna(how="any", inplace=True)
        temp_df = temp_df.iloc[:, :-1]
        temp_df.reset_index(inplace=True)
        temp_df.rename({"index": "date"}, axis="columns", inplace=True)
        temp_df["date"] = pd.to_datetime(temp_df["date"]).dt.date
        return temp_df

    if adjust == "hfq-factor":
        r = requests.get(hk_sina_stock_hist_hfq_url.format(symbol))
        hfq_factor_df = pd.DataFrame(eval(r.text.split("=")[1].split("\n")[0])["data"])
        hfq_factor_df.columns = ["date", "hfq_factor", "cash"]
        hfq_factor_df.index = pd.to_datetime(hfq_factor_df.date)
        del hfq_factor_df["date"]
        hfq_factor_df.reset_index(inplace=True)
        hfq_factor_df["date"] = pd.to_datetime(hfq_factor_df["date"]).dt.date
        return hfq_factor_df

    if adjust == "qfq-factor":
        r = requests.get(hk_sina_stock_hist_qfq_url.format(symbol))
        qfq_factor_df = pd.DataFrame(eval(r.text.split("=")[1].split("\n")[0])["data"])
        qfq_factor_df.columns = ["date", "qfq_factor"]
        qfq_factor_df.index = pd.to_datetime(qfq_factor_df.date)
        del qfq_factor_df["date"]
        qfq_factor_df.reset_index(inplace=True)
        qfq_factor_df["date"] = pd.to_datetime(qfq_factor_df["date"]).dt.date
        return qfq_factor_df


if __name__ == "__main__":
    stock_hk_daily_hfq_df = stock_hk_daily(symbol="00700", adjust="")
    print(stock_hk_daily_hfq_df)

    stock_hk_daily_hfq_df = stock_hk_daily(symbol="00700", adjust="hfq")
    print(stock_hk_daily_hfq_df)

    stock_hk_daily_hfq_df = stock_hk_daily(symbol="01591", adjust="hfq")
    print(stock_hk_daily_hfq_df)

    stock_hk_daily_hfq_df = stock_hk_daily(symbol="00700", adjust="qfq")
    print(stock_hk_daily_hfq_df)

    stock_hk_daily_df = stock_hk_daily(symbol="01302", adjust="qfq")
    print(stock_hk_daily_df)

    stock_hk_daily_hfq_factor_df = stock_hk_daily(symbol="00700", adjust="hfq-factor")
    print(stock_hk_daily_hfq_factor_df)

    stock_hk_spot_df = stock_hk_spot()
    print(stock_hk_spot_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2023/12/11 20:20
Desc: 东方财富网-指数行情数据
"""

from functools import lru_cache

import pandas as pd
import requests


@lru_cache()
def index_code_id_map_em() -> dict:
    """
    东方财富-股票和市场代码
    https://quote.eastmoney.com/center/gridlist.html#hs_a_board
    :return: 股票和市场代码
    :rtype: dict
    """
    url = "https://80.push2.eastmoney.com/api/qt/clist/get"
    params = {
        "pn": "1",
        "pz": "10000",
        "po": "1",
        "np": "1",
        "ut": "bd1d9ddb04089700cf9c27f6f7426281",
        "fltt": "2",
        "invt": "2",
        "fid": "f3",
        "fs": "m:1 t:2,m:1 t:23",
        "fields": "f12",
        "_": "1623833739532",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    if not data_json["data"]["diff"]:
        return dict()
    temp_df = pd.DataFrame(data_json["data"]["diff"])
    temp_df["market_id"] = 1
    temp_df.columns = ["sh_code", "sh_id"]
    code_id_dict = dict(zip(temp_df["sh_code"], temp_df["sh_id"]))
    params = {
        "pn": "1",
        "pz": "10000",
        "po": "1",
        "np": "1",
        "ut": "bd1d9ddb04089700cf9c27f6f7426281",
        "fltt": "2",
        "invt": "2",
        "fid": "f3",
        "fs": "m:0 t:6,m:0 t:80",
        "fields": "f12",
        "_": "1623833739532",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    if not data_json["data"]["diff"]:
        return dict()
    temp_df_sz = pd.DataFrame(data_json["data"]["diff"])
    temp_df_sz["sz_id"] = 0
    code_id_dict.update(dict(zip(temp_df_sz["f12"], temp_df_sz["sz_id"])))
    params = {
        "pn": "1",
        "pz": "10000",
        "po": "1",
        "np": "1",
        "ut": "bd1d9ddb04089700cf9c27f6f7426281",
        "fltt": "2",
        "invt": "2",
        "fid": "f3",
        "fs": "m:0 t:81 s:2048",
        "fields": "f12",
        "_": "1623833739532",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    if not data_json["data"]["diff"]:
        return dict()
    temp_df_sz = pd.DataFrame(data_json["data"]["diff"])
    temp_df_sz["bj_id"] = 0
    code_id_dict.update(dict(zip(temp_df_sz["f12"], temp_df_sz["bj_id"])))
    code_id_dict = {
        key: value - 1 if value == 1 else value + 1
        for key, value in code_id_dict.items()
    }
    return code_id_dict


def index_zh_a_hist(
    symbol: str = "000859",
    period: str = "daily",
    start_date: str = "19700101",
    end_date: str = "22220101",
) -> pd.DataFrame:
    """
    东方财富网-中国股票指数-行情数据
    https://quote.eastmoney.com/zz/2.000859.html
    :param symbol: 指数代码
    :type symbol: str
    :param period: choice of {'daily', 'weekly', 'monthly'}
    :type period: str
    :param start_date: 开始日期
    :type start_date: str
    :param end_date: 结束日期
    :type end_date: str
    :return: 行情数据
    :rtype: pandas.DataFrame
    """
    code_id_dict = index_code_id_map_em()
    period_dict = {"daily": "101", "weekly": "102", "monthly": "103"}
    url = "https://push2his.eastmoney.com/api/qt/stock/kline/get"
    try:
        params = {
            "secid": f"{code_id_dict[symbol]}.{symbol}",
            "ut": "7eea3edcaed734bea9cbfc24409ed989",
            "fields1": "f1,f2,f3,f4,f5,f6",
            "fields2": "f51,f52,f53,f54,f55,f56,f57,f58,f59,f60,f61",
            "klt": period_dict[period],
            "fqt": "0",
            "beg": "0",
            "end": "20500000",
            "_": "1623766962675",
        }
    except KeyError:
        params = {
            "secid": f"1.{symbol}",
            "ut": "7eea3edcaed734bea9cbfc24409ed989",
            "fields1": "f1,f2,f3,f4,f5,f6",
            "fields2": "f51,f52,f53,f54,f55,f56,f57,f58,f59,f60,f61",
            "klt": period_dict[period],
            "fqt": "0",
            "beg": "0",
            "end": "20500000",
            "_": "1623766962675",
        }
        r = requests.get(url, params=params)
        data_json = r.json()
        if data_json["data"] is None:
            params = {
                "secid": f"0.{symbol}",
                "ut": "7eea3edcaed734bea9cbfc24409ed989",
                "fields1": "f1,f2,f3,f4,f5,f6",
                "fields2": "f51,f52,f53,f54,f55,f56,f57,f58,f59,f60,f61",
                "klt": period_dict[period],
                "fqt": "0",
                "beg": "0",
                "end": "20500000",
                "_": "1623766962675",
            }
            r = requests.get(url, params=params)
            data_json = r.json()
            if data_json["data"] is None:
                params = {
                    "secid": f"2.{symbol}",
                    "ut": "7eea3edcaed734bea9cbfc24409ed989",
                    "fields1": "f1,f2,f3,f4,f5,f6",
                    "fields2": "f51,f52,f53,f54,f55,f56,f57,f58,f59,f60,f61",
                    "klt": period_dict[period],
                    "fqt": "0",
                    "beg": "0",
                    "end": "20500000",
                    "_": "1623766962675",
                }
                r = requests.get(url, params=params)
                data_json = r.json()
                if data_json["data"] is None:
                    params = {
                        "secid": f"47.{symbol}",
                        "ut": "7eea3edcaed734bea9cbfc24409ed989",
                        "fields1": "f1,f2,f3,f4,f5,f6",
                        "fields2": "f51,f52,f53,f54,f55,f56,f57,f58,f59,f60,f61",
                        "klt": period_dict[period],
                        "fqt": "0",
                        "beg": "0",
                        "end": "20500000",
                        "_": "1623766962675",
                    }
    r = requests.get(url, params=params)
    data_json = r.json()
    try:
        temp_df = pd.DataFrame(
            [item.split(",") for item in data_json["data"]["klines"]]
        )
    except:  # noqa: E722
        # 兼容 000859(中证国企一路一带) 和 000861(中证央企创新)
        params = {
            "secid": f"2.{symbol}",
            "ut": "7eea3edcaed734bea9cbfc24409ed989",
            "fields1": "f1,f2,f3,f4,f5,f6",
            "fields2": "f51,f52,f53,f54,f55,f56,f57,f58,f59,f60,f61",
            "klt": period_dict[period],
            "fqt": "0",
            "beg": "0",
            "end": "20500000",
            "_": "1623766962675",
        }
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(
            [item.split(",") for item in data_json["data"]["klines"]]
        )
    temp_df.columns = [
        "日期",
        "开盘",
        "收盘",
        "最高",
        "最低",
        "成交量",
        "成交额",
        "振幅",
        "涨跌幅",
        "涨跌额",
        "换手率",
    ]
    temp_df.index = pd.to_datetime(temp_df["日期"], errors="coerce")
    temp_df = temp_df[start_date:end_date]
    temp_df.reset_index(inplace=True, drop=True)
    temp_df["开盘"] = pd.to_numeric(temp_df["开盘"], errors="coerce")
    temp_df["收盘"] = pd.to_numeric(temp_df["收盘"], errors="coerce")
    temp_df["最高"] = pd.to_numeric(temp_df["最高"], errors="coerce")
    temp_df["最低"] = pd.to_numeric(temp_df["最低"], errors="coerce")
    temp_df["成交量"] = pd.to_numeric(temp_df["成交量"], errors="coerce")
    temp_df["成交额"] = pd.to_numeric(temp_df["成交额"], errors="coerce")
    temp_df["振幅"] = pd.to_numeric(temp_df["振幅"], errors="coerce")
    temp_df["涨跌幅"] = pd.to_numeric(temp_df["涨跌幅"], errors="coerce")
    temp_df["涨跌额"] = pd.to_numeric(temp_df["涨跌额"], errors="coerce")
    temp_df["换手率"] = pd.to_numeric(temp_df["换手率"], errors="coerce")
    return temp_df


def index_zh_a_hist_min_em(
    symbol: str = "399006",
    period: str = "1",
    start_date: str = "1979-09-01 09:32:00",
    end_date: str = "2222-01-01 09:32:00",
) -> pd.DataFrame:
    """
    东方财富网-指数数据-每日分时行情
    https://quote.eastmoney.com/center/hszs.html
    :param symbol: 指数代码
    :type symbol: str
    :param period: choice of {'1', '5', '15', '30', '60'}
    :type period: str
    :param start_date: 开始日期
    :type start_date: str
    :param end_date: 结束日期
    :type end_date: str
    :return: 每日分时行情
    :rtype: pandas.DataFrame
    """
    code_id_dict = index_code_id_map_em()
    if period == "1":
        url = "https://push2his.eastmoney.com/api/qt/stock/trends2/get"
        try:
            params = {
                "fields1": "f1,f2,f3,f4,f5,f6,f7,f8,f9,f10,f11,f12,f13",
                "fields2": "f51,f52,f53,f54,f55,f56,f57,f58",
                "ut": "fa5fd1943c7b386f172d6893dbfba10b",
                "iscr": "0",
                "ndays": "5",
                "secid": f"{code_id_dict[symbol]}.{symbol}",
                "_": "1623766962675",
            }
        except KeyError:
            params = {
                "fields1": "f1,f2,f3,f4,f5,f6,f7,f8,f9,f10,f11,f12,f13",
                "fields2": "f51,f52,f53,f54,f55,f56,f57,f58",
                "ut": "fa5fd1943c7b386f172d6893dbfba10b",
                "iscr": "0",
                "ndays": "5",
                "secid": f"1.{symbol}",
                "_": "1623766962675",
            }
            r = requests.get(url, params=params)
            data_json = r.json()
            if data_json["data"] is None:
                params = {
                    "fields1": "f1,f2,f3,f4,f5,f6,f7,f8,f9,f10,f11,f12,f13",
                    "fields2": "f51,f52,f53,f54,f55,f56,f57,f58",
                    "ut": "fa5fd1943c7b386f172d6893dbfba10b",
                    "iscr": "0",
                    "ndays": "5",
                    "secid": f"0.{symbol}",
                    "_": "1623766962675",
                }
                r = requests.get(url, params=params)
                data_json = r.json()
                if data_json["data"] is None:
                    params = {
                        "fields1": "f1,f2,f3,f4,f5,f6,f7,f8,f9,f10,f11,f12,f13",
                        "fields2": "f51,f52,f53,f54,f55,f56,f57,f58",
                        "ut": "fa5fd1943c7b386f172d6893dbfba10b",
                        "iscr": "0",
                        "ndays": "5",
                        "secid": f"47.{symbol}",
                        "_": "1623766962675",
                    }
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(
            [item.split(",") for item in data_json["data"]["trends"]]
        )
        temp_df.columns = [
            "时间",
            "开盘",
            "收盘",
            "最高",
            "最低",
            "成交量",
            "成交额",
            "最新价",
        ]
        temp_df.index = pd.to_datetime(temp_df["时间"], errors="coerce")
        temp_df = temp_df[start_date:end_date]
        temp_df.reset_index(drop=True, inplace=True)
        temp_df["开盘"] = pd.to_numeric(temp_df["开盘"], errors="coerce")
        temp_df["收盘"] = pd.to_numeric(temp_df["收盘"], errors="coerce")
        temp_df["最高"] = pd.to_numeric(temp_df["最高"], errors="coerce")
        temp_df["最低"] = pd.to_numeric(temp_df["最低"], errors="coerce")
        temp_df["成交量"] = pd.to_numeric(temp_df["成交量"], errors="coerce")
        temp_df["成交额"] = pd.to_numeric(temp_df["成交额"], errors="coerce")
        temp_df["最新价"] = pd.to_numeric(temp_df["最新价"], errors="coerce")
        temp_df["时间"] = pd.to_datetime(temp_df["时间"]).astype(str)
        return temp_df
    else:
        url = "http://push2his.eastmoney.com/api/qt/stock/kline/get"
        try:
            params = {
                "secid": f"{code_id_dict[symbol]}.{symbol}",
                "ut": "7eea3edcaed734bea9cbfc24409ed989",
                "fields1": "f1,f2,f3,f4,f5,f6",
                "fields2": "f51,f52,f53,f54,f55,f56,f57,f58,f59,f60,f61",
                "klt": period,
                "fqt": "1",
                "beg": "0",
                "end": "20500000",
                "_": "1630930917857",
            }
        except:  # noqa: E722
            params = {
                "secid": f"0.{symbol}",
                "ut": "7eea3edcaed734bea9cbfc24409ed989",
                "fields1": "f1,f2,f3,f4,f5,f6",
                "fields2": "f51,f52,f53,f54,f55,f56,f57,f58,f59,f60,f61",
                "klt": period,
                "fqt": "1",
                "beg": "0",
                "end": "20500000",
                "_": "1630930917857",
            }
            r = requests.get(url, params=params)
            data_json = r.json()
            if data_json["data"] is None:
                params = {
                    "secid": f"1.{symbol}",
                    "ut": "7eea3edcaed734bea9cbfc24409ed989",
                    "fields1": "f1,f2,f3,f4,f5,f6",
                    "fields2": "f51,f52,f53,f54,f55,f56,f57,f58,f59,f60,f61",
                    "klt": period,
                    "fqt": "1",
                    "beg": "0",
                    "end": "20500000",
                    "_": "1630930917857",
                }
                r = requests.get(url, params=params)
                data_json = r.json()
                if data_json["data"] is None:
                    params = {
                        "secid": f"47.{symbol}",
                        "ut": "7eea3edcaed734bea9cbfc24409ed989",
                        "fields1": "f1,f2,f3,f4,f5,f6",
                        "fields2": "f51,f52,f53,f54,f55,f56,f57,f58,f59,f60,f61",
                        "klt": period,
                        "fqt": "1",
                        "beg": "0",
                        "end": "20500000",
                        "_": "1630930917857",
                    }
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(
            [item.split(",") for item in data_json["data"]["klines"]]
        )
        temp_df.columns = [
            "时间",
            "开盘",
            "收盘",
            "最高",
            "最低",
            "成交量",
            "成交额",
            "振幅",
            "涨跌幅",
            "涨跌额",
            "换手率",
        ]
        temp_df.index = pd.to_datetime(temp_df["时间"], errors="coerce")
        temp_df = temp_df[start_date:end_date]
        temp_df.reset_index(drop=True, inplace=True)
        temp_df["开盘"] = pd.to_numeric(temp_df["开盘"], errors="coerce")
        temp_df["收盘"] = pd.to_numeric(temp_df["收盘"], errors="coerce")
        temp_df["最高"] = pd.to_numeric(temp_df["最高"], errors="coerce")
        temp_df["最低"] = pd.to_numeric(temp_df["最低"], errors="coerce")
        temp_df["成交量"] = pd.to_numeric(temp_df["成交量"], errors="coerce")
        temp_df["成交额"] = pd.to_numeric(temp_df["成交额"], errors="coerce")
        temp_df["振幅"] = pd.to_numeric(temp_df["振幅"], errors="coerce")
        temp_df["涨跌幅"] = pd.to_numeric(temp_df["涨跌幅"], errors="coerce")
        temp_df["涨跌额"] = pd.to_numeric(temp_df["涨跌额"], errors="coerce")
        temp_df["换手率"] = pd.to_numeric(temp_df["换手率"], errors="coerce")
        temp_df["时间"] = pd.to_datetime(temp_df["时间"], errors="coerce").astype(str)
        temp_df = temp_df[
            [
                "时间",
                "开盘",
                "收盘",
                "最高",
                "最低",
                "涨跌幅",
                "涨跌额",
                "成交量",
                "成交额",
                "振幅",
                "换手率",
            ]
        ]
        return temp_df


if __name__ == "__main__":
    index_zh_a_hist_df = index_zh_a_hist(
        symbol="800000",
        period="daily",
        start_date="19700101",
        end_date="22220101",
    )
    print(index_zh_a_hist_df)

    index_zh_a_hist_min_em_df = index_zh_a_hist_min_em(
        symbol="000001",
        period="1",
        start_date="2024-03-20 09:30:00",
        end_date="2024-03-20 19:00:00",
    )
    print(index_zh_a_hist_min_em_df)



"""
AKShare 异常处理模块
"""


class AkshareException(Exception):
    """Base exception for byteapp_ak library"""

    def __init__(self, message):
        self.message = message
        super().__init__(self.message)


class APIError(AkshareException):
    """Raised when API request fails"""

    def __init__(self, message, status_code=None):
        self.status_code = status_code
        super().__init__(f"API Error: {message} (Status code: {status_code})")


class DataParsingError(AkshareException):
    """Raised when data parsing fails"""

    pass


class InvalidParameterError(AkshareException):
    """Raised when an invalid parameter is provided"""

    pass


class NetworkError(AkshareException):
    """Raised when network-related issues occur"""

    pass


class RateLimitError(AkshareException):
    """Raised when API rate limit is exceeded"""

    pass



# -*- coding:utf-8 -*-
# !/usr/bin/env python
"""
Date: 2024/1/7 17:00
Desc: 雪球-沪深股市-热度排行榜
https://xueqiu.com/hq
"""
import math

import pandas as pd
import requests

from byteapp_ak.utils.tqdm import get_tqdm


def stock_hot_follow_xq(symbol: str = "最热门") -> pd.DataFrame:
    """
    雪球-沪深股市-热度排行榜-关注排行榜
    https://xueqiu.com/hq
    :param symbol: choice of {"本周新增", "最热门"}
    :type symbol: str
    :return: 关注排行榜
    :rtype: pandas.DataFrame
    """
    symbol_map = {
        "本周新增": "follow7d",
        "最热门": "follow",
    }
    url = "https://xueqiu.com/service/v5/stock/screener/screen"
    params = {
        "category": "CN",
        "size": "200",
        "order": "desc",
        "order_by": symbol_map[symbol],
        "only_count": "0",
        "page": "1",
        "_": "1651050034006",
    }
    headers = {
        "Accept": "*/*",
        "Accept-Encoding": "gzip, deflate, br",
        "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
        "Cache-Control": "no-cache",
        "Connection": "keep-alive",
        "Host": "xueqiu.com",
        "Pragma": "no-cache",
        "Referer": "https://xueqiu.com/hq",
        "sec-ch-ua": '" Not A;Brand";v="99", "Chromium";v="100", "Google Chrome";v="100"',
        "sec-ch-ua-mobile": "?0",
        "sec-ch-ua-platform": '"Windows"',
        "Sec-Fetch-Dest": "empty",
        "Sec-Fetch-Mode": "cors",
        "Sec-Fetch-Site": "same-origin",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/100.0.4896.127 Safari/537.36",
        "X-Requested-With": "XMLHttpRequest",
    }
    r = requests.get(url, params=params, headers=headers)
    data_json = r.json()
    total_num = data_json["data"]['count']
    total_page = math.ceil(total_num / 200)
    tqdm = get_tqdm()
    big_df = pd.DataFrame()
    for page in tqdm(range(1, total_page + 1), leave=False):
        params.update({"page": page})
        r = requests.get(url, params=params, headers=headers)
        data_json = r.json()
        try:
            temp_df = pd.DataFrame(data_json["data"]["list"])
        except TypeError:
            temp_df = pd.DataFrame()
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)
    if symbol == "本周新增":
        big_df = big_df[
            [
                "symbol",
                "name",
                "follow7d",
                "current",
            ]
        ]
    else:
        big_df = big_df[
            [
                "symbol",
                "name",
                "follow",
                "current",
            ]
        ]
    big_df.columns = [
        "股票代码",
        "股票简称",
        "关注",
        "最新价",
    ]
    big_df["关注"] = pd.to_numeric(big_df["关注"], errors="coerce")
    big_df["最新价"] = pd.to_numeric(big_df["最新价"], errors="coerce")
    return big_df


def stock_hot_tweet_xq(symbol: str = "最热门") -> pd.DataFrame:
    """
    雪球-沪深股市-热度排行榜-讨论排行榜
    https://xueqiu.com/hq
    :param symbol: choice of {"本周新增", "最热门"}
    :type symbol: str
    :return: 讨论排行榜
    :rtype: pandas.DataFrame
    """
    symbol_map = {
        "本周新增": "tweet7d",
        "最热门": "tweet",
    }
    url = "https://xueqiu.com/service/v5/stock/screener/screen"
    params = {
        "category": "CN",
        "size": "200",
        "order": "desc",
        "order_by": symbol_map[symbol],
        "only_count": "0",
        "page": "1",
        "_": "1651050034006",
    }
    headers = {
        "Accept": "*/*",
        "Accept-Encoding": "gzip, deflate, br",
        "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
        "Cache-Control": "no-cache",
        "Connection": "keep-alive",
        "Host": "xueqiu.com",
        "Pragma": "no-cache",
        "Referer": "https://xueqiu.com/hq",
        "sec-ch-ua": '" Not A;Brand";v="99", "Chromium";v="100", "Google Chrome";v="100"',
        "sec-ch-ua-mobile": "?0",
        "sec-ch-ua-platform": '"Windows"',
        "Sec-Fetch-Dest": "empty",
        "Sec-Fetch-Mode": "cors",
        "Sec-Fetch-Site": "same-origin",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/100.0.4896.127 Safari/537.36",
        "X-Requested-With": "XMLHttpRequest",
    }
    r = requests.get(url, params=params, headers=headers)
    data_json = r.json()
    total_num = data_json["data"]['count']
    total_page = math.ceil(total_num / 200)
    tqdm = get_tqdm()
    big_df = pd.DataFrame()
    for page in tqdm(range(1, total_page + 1), leave=False):
        params.update({"page": page})
        r = requests.get(url, params=params, headers=headers)
        data_json = r.json()
        try:
            temp_df = pd.DataFrame(data_json["data"]["list"])
        except TypeError:
            temp_df = pd.DataFrame()
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)
    if symbol == "本周新增":
        big_df = big_df[
            [
                "symbol",
                "name",
                "tweet7d",
                "current",
            ]
        ]
    else:
        big_df = big_df[
            [
                "symbol",
                "name",
                "tweet",
                "current",
            ]
        ]
    big_df.columns = [
        "股票代码",
        "股票简称",
        "关注",
        "最新价",
    ]
    big_df["关注"] = pd.to_numeric(big_df["关注"], errors="coerce")
    big_df["最新价"] = pd.to_numeric(big_df["最新价"], errors="coerce")
    return big_df


def stock_hot_deal_xq(symbol: str = "最热门") -> pd.DataFrame:
    """
    雪球-沪深股市-热度排行榜-分享交易排行榜
    https://xueqiu.com/hq
    :param symbol: choice of {"本周新增", "最热门"}
    :type symbol: str
    :return: 分享交易排行榜
    :rtype: pandas.DataFrame
    """
    symbol_map = {
        "本周新增": "deal7d",
        "最热门": "deal",
    }
    url = "https://xueqiu.com/service/v5/stock/screener/screen"
    params = {
        "category": "CN",
        "size": "10000",
        "order": "desc",
        "order_by": symbol_map[symbol],
        "only_count": "0",
        "page": "1",
        "_": "1651050034006",
    }
    headers = {
        "Accept": "*/*",
        "Accept-Encoding": "gzip, deflate, br",
        "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
        "Cache-Control": "no-cache",
        "Connection": "keep-alive",
        "Host": "xueqiu.com",
        "Pragma": "no-cache",
        "Referer": "https://xueqiu.com/hq",
        "sec-ch-ua": '" Not A;Brand";v="99", "Chromium";v="100", "Google Chrome";v="100"',
        "sec-ch-ua-mobile": "?0",
        "sec-ch-ua-platform": '"Windows"',
        "Sec-Fetch-Dest": "empty",
        "Sec-Fetch-Mode": "cors",
        "Sec-Fetch-Site": "same-origin",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/100.0.4896.127 Safari/537.36",
        "X-Requested-With": "XMLHttpRequest",
    }
    r = requests.get(url, params=params, headers=headers)
    data_json = r.json()
    total_num = data_json["data"]['count']
    total_page = math.ceil(total_num / 200)
    tqdm = get_tqdm()
    big_df = pd.DataFrame()
    for page in tqdm(range(1, total_page + 1), leave=False):
        params.update({"page": page})
        r = requests.get(url, params=params, headers=headers)
        data_json = r.json()
        try:
            temp_df = pd.DataFrame(data_json["data"]["list"])
        except TypeError:
            temp_df = pd.DataFrame()
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)
    if symbol == "本周新增":
        big_df = big_df[
            [
                "symbol",
                "name",
                "deal7d",
                "current",
            ]
        ]
    else:
        big_df = big_df[
            [
                "symbol",
                "name",
                "deal",
                "current",
            ]
        ]
    big_df.columns = [
        "股票代码",
        "股票简称",
        "关注",
        "最新价",
    ]
    big_df["关注"] = pd.to_numeric(big_df["关注"], errors="coerce")
    big_df["最新价"] = pd.to_numeric(big_df["最新价"], errors="coerce")
    return big_df


if __name__ == "__main__":
    stock_hot_follow_xq_df = stock_hot_follow_xq(symbol="本周新增")
    print(stock_hot_follow_xq_df)

    stock_hot_follow_xq_df = stock_hot_follow_xq(symbol="最热门")
    print(stock_hot_follow_xq_df)

    stock_hot_tweet_xq_df = stock_hot_tweet_xq(symbol="本周新增")
    print(stock_hot_tweet_xq_df)

    stock_hot_tweet_xq_df = stock_hot_tweet_xq(symbol="最热门")
    print(stock_hot_tweet_xq_df)

    stock_hot_deal_xq_df = stock_hot_deal_xq(symbol="本周新增")
    print(stock_hot_deal_xq_df)

    stock_hot_deal_xq_df = stock_hot_deal_xq(symbol="最热门")
    print(stock_hot_deal_xq_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2022/11/27 13:30
Desc: 美股目标价 or 港股目标价
https://www.ushknews.com/report.html
"""
import requests
import pandas as pd


def stock_price_js(symbol: str = "us") -> pd.DataFrame:
    """
    美股目标价 or 港股目标价
    https://www.ushknews.com/report.html
    :param symbol: choice of {"us", "hk"}
    :type symbol: str
    :return: 美股目标价 or 港股目标价
    :rtype: pandas.DataFrame
    """
    url = "https://calendar-api.ushknews.com/getWebTargetPriceList"
    params = {
        'limit': '20',
        'category': symbol,
    }
    headers = {
        "accept": "application/json, text/plain, */*",
        "accept-encoding": "gzip, deflate, br",
        "accept-language": "zh-CN,zh;q=0.9,en;q=0.8",
        "cache-control": "no-cache",
        "origin": "https://www.ushknews.com",
        "pragma": "no-cache",
        "referer": "https://www.ushknews.com/",
        "sec-ch-ua": '"Google Chrome";v="107", "Chromium";v="107", "Not=A?Brand";v="24"',
        "sec-ch-ua-mobile": "?0",
        "sec-ch-ua-platform": '"Windows"',
        "sec-fetch-dest": "empty",
        "sec-fetch-mode": "cors",
        "sec-fetch-site": "same-site",
        "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/107.0.0.0 Safari/537.36",
        "x-app-id": "BNsiR9uq7yfW0LVz",
        "x-version": "1.0.0",
    }
    r = requests.get(url, params=params, headers=headers)
    json_data = r.json()
    temp_df = pd.DataFrame(json_data["data"]["list"])
    temp_df.columns = [
        "_",
        "_",
        "评级",
        "_",
        "最新目标价",
        "先前目标价",
        "机构名称",
        "日期",
        "_",
        "个股名称",
        "_",
        "_",
    ]
    temp_df = temp_df[
        [
            "日期",
            "个股名称",
            "评级",
            "先前目标价",
            "最新目标价",
            "机构名称",
        ]
    ]
    temp_df["日期"] = pd.to_datetime(temp_df["日期"]).dt.date
    temp_df["先前目标价"] = pd.to_numeric(temp_df["先前目标价"], errors="coerce")
    temp_df["最新目标价"] = pd.to_numeric(temp_df["最新目标价"], errors="coerce")
    return temp_df


if __name__ == "__main__":
    stock_price_js_df = stock_price_js(symbol="us")
    print(stock_price_js_df)

    stock_price_js_df = stock_price_js(symbol="hk")
    print(stock_price_js_df)



# -*- coding:utf-8 -*-
# !/usr/bin/env python
"""
Date: 2024/9/3 15:30
Desc: 导入文件工具，可以正确处理路径问题
"""

import pathlib
from importlib import resources


def get_ths_js(file: str = "ths.js") -> pathlib.Path:
    """
    get path to data "ths.js" text file.
    :return: 文件路径
    :rtype: pathlib.Path
    """
    with resources.path(package="byteapp_ak.data", resource=file) as f:
        data_file_path = f
        return data_file_path


def get_crypto_info_csv(file: str = "crypto_info.zip") -> pathlib.Path:
    """
    get path to data "ths.js" text file.
    :return: 文件路径
    :rtype: pathlib.Path
    """
    with resources.path(package="byteapp_ak.data", resource=file) as f:
        data_file_path = f
        return data_file_path


if __name__ == "__main__":
    get_ths_js_path = get_ths_js(file="ths.js")
    print(get_ths_js_path)

    get_crypto_info_csv_path = get_crypto_info_csv(file="crypto_info.zip")
    print(get_crypto_info_csv_path)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/4/3 19:00
Desc: 汽车行业制造企业数据库
http://i.gasgoo.com/data/ranking
"""

import pandas as pd
import requests

from byteapp_ak.utils import demjson


def car_sale_rank_gasgoo(symbol: str = "车企榜", date: str = "202109") -> pd.DataFrame:
    """
    盖世汽车-汽车行业制造企业数据库-销量数据
    https://i.gasgoo.com/data/ranking
    :param symbol: choice of {"车企榜", "品牌榜", "车型榜"}
    :type symbol: str
    :param date: 查询的年份和月份
    :type date: str
    :return: 销量数据
    :rtype: pandas.DataFrame
    """
    symbol_map = {
        "车型榜": "M",
        "车企榜": "F",
        "品牌榜": "B",
    }
    url = "https://i.gasgoo.com/data/sales/AutoModelSalesRank.aspx/GetSalesRank"
    payload = {
        "countryID": "",
        "endM": str(int(date[4:6])),
        "endY": date[:4],
        "energy": "",
        "modelGradeID": "",
        "modelTypeID": "",
        "orderBy": f"{date[:4]}-{str(int(date[4:6]))}",
        "queryDate": f"{date[:4]}-{str(int(date[4:6]))}",
        "rankType": symbol_map[symbol],
        "startY": date[:4],
        "startM": str(int(date[4:6])),
    }
    headers = {
        "Accept": "application/json, text/javascript, */*; q=0.01",
        "Accept-Encoding": "gzip, deflate, br",
        "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
        "Cache-Control": "no-cache",
        "Connection": "keep-alive",
        "Content-Length": "195",
        "Content-Type": "application/json; charset=UTF-8",
        "Cookie": "Hm_lvt_8e90480b1bf68ede548c407057660718=1636981448; _ga=GA1.2.858318653.1636981449; "
        "_gid=GA1.2.1448165285.1636981449; _gat=1; Hm_lpvt_8e90480b1bf68ede548c407057660718=1636982578",
        "Host": "i.gasgoo.com",
        "Origin": "https://i.gasgoo.com",
        "Pragma": "no-cache",
        "Referer": "https://i.gasgoo.com/data/sales/AutoModelSalesRank.aspx/GetSalesRank",
        "sec-ch-ua": '"Google Chrome";v="95", "Chromium";v="95", ";Not A Brand";v="99"',
        "sec-ch-ua-mobile": "?0",
        "sec-ch-ua-platform": '"Windows"',
        "Sec-Fetch-Dest": "empty",
        "Sec-Fetch-Mode": "cors",
        "Sec-Fetch-Site": "same-origin",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/95.0.4638.69 Safari/537.36",
        "X-Requested-With": "XMLHttpRequest",
    }
    r = requests.post(url, json=payload, headers=headers)
    data_json = r.json()
    data_json = demjson.decode(data_json["d"])
    temp_df = pd.DataFrame(data_json)
    return temp_df


if __name__ == "__main__":
    car_sale_rank_gasgoo_df = car_sale_rank_gasgoo(symbol="品牌榜", date="202311")
    print(car_sale_rank_gasgoo_df)

    car_sale_rank_gasgoo_df = car_sale_rank_gasgoo(symbol="车型榜", date="202311")
    print(car_sale_rank_gasgoo_df)

    car_sale_rank_gasgoo_df = car_sale_rank_gasgoo(symbol="车企榜", date="202311")
    print(car_sale_rank_gasgoo_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2022/5/11 17:52
Desc: 加密货币
https://cn.investing.com/crypto/currencies
高频数据
https://bitcoincharts.com/about/markets-api/
"""
import math

import pandas as pd
import requests
from tqdm import tqdm

from byteapp_ak.datasets import get_crypto_info_csv


def crypto_name_url_table(symbol: str = "web") -> pd.DataFrame:
    """
    加密货币名称、代码和 ID，每次更新较慢
    https://cn.investing.com/crypto/ethereum/historical-data
    :param symbol: choice of {"web", "local"}; web 表示从网页获取最新，local 表示利用本地本文件
    :type symbol: str
    :return: 加密货币名称、代码和 ID
    :rtype: pandas.DataFrame
    """
    if symbol == "web":
        headers = {
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/86.0.4240.111 Safari/537.36",
            "X-Requested-With": "XMLHttpRequest",
        }
        url = "https://cn.investing.com/crypto/Service/LoadCryptoCurrencies"
        payload = {
            'draw': '14',
            'columns[0][data]': 'currencies_order',
            'columns[0][name]': 'currencies_order',
            'columns[0][searchable]': 'true',
            'columns[0][orderable]': 'true',
            'columns[0][search][value]': '',
            'columns[0][search][regex]': 'false',
            'columns[1][data]': 'function',
            'columns[1][name]': 'crypto_id',
            'columns[1][searchable]': 'true',
            'columns[1][orderable]': 'false',
            'columns[1][search][value]': '',
            'columns[1][search][regex]': 'false',
            'columns[2][data]': 'function',
            'columns[2][name]': 'name',
            'columns[2][searchable]': 'true',
            'columns[2][orderable]': 'true',
            'columns[2][search][value]': '',
            'columns[2][search][regex]': 'false',
            'columns[3][data]': 'symbol',
            'columns[3][name]': 'symbol',
            'columns[3][searchable]': 'true',
            'columns[3][orderable]': 'true',
            'columns[3][search][value]': '',
            'columns[3][search][regex]': 'false',
            'columns[4][data]': 'function',
            'columns[4][name]': 'price_usd',
            'columns[4][searchable]': 'true',
            'columns[4][orderable]': 'true',
            'columns[4][search][value]': '',
            'columns[4][search][regex]': 'false',
            'columns[5][data]': 'market_cap_formatted',
            'columns[5][name]': 'market_cap_usd',
            'columns[5][searchable]': 'true',
            'columns[5][orderable]': 'true',
            'columns[5][search][value]': '',
            'columns[5][search][regex]': 'false',
            'columns[6][data]': '24h_volume_formatted',
            'columns[6][name]': '24h_volume_usd',
            'columns[6][searchable]': 'true',
            'columns[6][orderable]': 'true',
            'columns[6][search][value]': '',
            'columns[6][search][regex]': 'false',
            'columns[7][data]': 'total_volume',
            'columns[7][name]': 'total_volume',
            'columns[7][searchable]': 'true',
            'columns[7][orderable]': 'true',
            'columns[7][search][value]': '',
            'columns[7][search][regex]': 'false',
            'columns[8][data]': 'change_percent_formatted',
            'columns[8][name]': 'change_percent',
            'columns[8][searchable]': 'true',
            'columns[8][orderable]': 'true',
            'columns[8][search][value]': '',
            'columns[8][search][regex]': 'false',
            'columns[9][data]': 'percent_change_7d_formatted',
            'columns[9][name]': 'percent_change_7d',
            'columns[9][searchable]': 'true',
            'columns[9][orderable]': 'true',
            'columns[9][search][value]': '',
            'columns[9][search][regex]': 'false',
            'order[0][column]': 'currencies_order',
            'order[0][dir]': 'asc',
            'start': '0',
            'length': '100',
            'search[value]': '',
            'search[regex]': 'false',
            'currencyId': '12',
        }
        r = requests.post(url, data=payload, headers=headers)
        data_json = r.json()
        total_page = math.ceil(int(data_json['recordsTotal']) / 100)
        big_df = pd.DataFrame()
        for page in tqdm(range(1, total_page+1), leave=False):
            payload.update({
                "start": (page-1)*100,
                'length': 100
            })
            r = requests.post(url, data=payload, headers=headers)
            data_json = r.json()
            temp_df = pd.DataFrame(data_json['data'])
            big_df = pd.concat([big_df, temp_df], ignore_index=True)
        big_df = big_df[[
            'symbol',
            'name',
            'name_trans',
            'sml_id',
            'related_pair_ID',
        ]]
        return big_df
    else:
        get_crypto_info_csv_path = get_crypto_info_csv()
        name_url_df = pd.read_csv(get_crypto_info_csv_path)
        return name_url_df


def crypto_hist(
    symbol: str = "BTC",
    period: str = "每日",
    start_date: str = "20191020",
    end_date: str = "20201020",
):
    """
    加密货币历史数据
    https://cn.investing.com/crypto/ethereum/historical-data
    :param symbol: 货币名称
    :type symbol: str
    :param period: choice of {"每日", "每周", "每月"}
    :type period: str
    :param start_date: '20151020', 注意格式
    :type start_date: str
    :param end_date: '20201020', 注意格式
    :type end_date: str
    :return: 加密货币历史数据获取
    :rtype: pandas.DataFrame
    """
    import warnings
    warnings.filterwarnings('ignore')
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/86.0.4240.111 Safari/537.36",
        "X-Requested-With": "XMLHttpRequest",
    }
    period_map = {"每日": "Daily", "每周": "Weekly", "每月": "Monthly"}
    start_date = "/".join([start_date[:4], start_date[4:6], start_date[6:]])
    end_date = "/".join([end_date[:4], end_date[4:6], end_date[6:]])
    name_url_df = crypto_name_url_table(symbol='local')
    curr_id = name_url_df[name_url_df["symbol"] == symbol]["related_pair_ID"].values[0]
    sml_id = name_url_df[name_url_df["symbol"] == symbol]["sml_id"].values[0]
    url = "https://cn.investing.com/instruments/HistoricalDataAjax"
    payload = {
        "curr_id": curr_id,
        "smlID": sml_id,
        "header": "null",
        "st_date": start_date,
        "end_date": end_date,
        "interval_sec": period_map[period],
        "sort_col": "date",
        "sort_ord": "DESC",
        "action": "historical_data",
    }
    r = requests.post(url, data=payload, headers=headers)

    temp_df = pd.read_html(r.text)[0]
    df_data = temp_df.copy()
    if period == "每月":
        df_data.index = pd.to_datetime(df_data["日期"], format="%Y年%m月")
    else:
        df_data.index = pd.to_datetime(df_data["日期"], format="%Y年%m月%d日")
    if any(df_data["交易量"].astype(str).str.contains("-")):
        df_data["交易量"][df_data["交易量"].str.contains("-")] = df_data["交易量"][
            df_data["交易量"].str.contains("-")
        ].replace("-", 0)
    if any(df_data["交易量"].astype(str).str.contains("B")):
        df_data["交易量"][df_data["交易量"].str.contains("B").fillna(False)] = (
            df_data["交易量"][df_data["交易量"].str.contains("B").fillna(False)]
            .str.replace("B", "")
            .str.replace(",", "")
            .astype(float)
            * 1000000000
        )
    if any(df_data["交易量"].astype(str).str.contains("M")):
        df_data["交易量"][df_data["交易量"].str.contains("M").fillna(False)] = (
            df_data["交易量"][df_data["交易量"].str.contains("M").fillna(False)]
            .str.replace("M", "")
            .str.replace(",", "")
            .astype(float)
            * 1000000
        )
    if any(df_data["交易量"].astype(str).str.contains("K")):
        df_data["交易量"][df_data["交易量"].str.contains("K").fillna(False)] = (
            df_data["交易量"][df_data["交易量"].str.contains("K").fillna(False)]
            .str.replace("K", "")
            .str.replace(",", "")
            .astype(float)
            * 1000
        )
    df_data["交易量"] = df_data["交易量"].astype(float)
    df_data["涨跌幅"] = pd.DataFrame(
        round(
            df_data["涨跌幅"].str.replace(",", "").str.replace("%", "").astype(float)
            / 100,
            6,
        )
    )
    del df_data["日期"]
    df_data.reset_index(inplace=True)
    df_data = df_data[[
        "日期",
        "收盘",
        "开盘",
        "高",
        "低",
        "交易量",
        "涨跌幅",
    ]]
    df_data['日期'] = pd.to_datetime(df_data['日期']).dt.date
    df_data['收盘'] = pd.to_numeric(df_data['收盘'])
    df_data['开盘'] = pd.to_numeric(df_data['开盘'])
    df_data['高'] = pd.to_numeric(df_data['高'])
    df_data['低'] = pd.to_numeric(df_data['低'])
    df_data['交易量'] = pd.to_numeric(df_data['交易量'])
    df_data['涨跌幅'] = pd.to_numeric(df_data['涨跌幅'])
    df_data.sort_values('日期', inplace=True)
    df_data.reset_index(inplace=True, drop=True)
    return df_data


if __name__ == "__main__":
    crypto_name_url_table_df = crypto_name_url_table(symbol="local")
    print(crypto_name_url_table_df)

    crypto_hist_df = crypto_hist(
        symbol="BTC", period="每日", start_date="20201020", end_date="20220511"
    )
    print(crypto_hist_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/3/20 15:00
Desc: 债券-集思录-可转债
集思录：https://app.jisilu.cn/data/cbnew/#cb
"""

from io import StringIO
import pandas as pd
import requests

from byteapp_ak.utils import demjson


def bond_cb_index_jsl() -> pd.DataFrame:
    """
    首页-可转债-集思录可转债等权指数
    https://www.jisilu.cn/web/data/cb/index
    :return: 集思录可转债等权指数
    :rtype: pandas.DataFrame
    """
    url = "https://www.jisilu.cn/webapi/cb/index_history/"
    r = requests.get(url)
    data_dict = demjson.decode(r.text)["data"]
    temp_df = pd.DataFrame(data_dict)
    return temp_df


def bond_cb_jsl(cookie: str = None) -> pd.DataFrame:
    """
    集思录可转债
    https://app.jisilu.cn/data/cbnew/#cb
    :param cookie: 输入获取到的游览器 cookie
    :type cookie: str
    :return: 集思录可转债
    :rtype: pandas.DataFrame
    """
    url = "https://app.jisilu.cn/data/cbnew/cb_list_new/"
    headers = {
        "accept": "application/json, text/javascript, */*; q=0.01",
        "accept-encoding": "gzip, deflate, br",
        "accept-language": "zh-CN,zh;q=0.9,en;q=0.8",
        "cache-control": "no-cache",
        "content-length": "220",
        "content-type": "application/x-www-form-urlencoded; charset=UTF-8",
        "cookie": cookie,
        "origin": "https://app.jisilu.cn",
        "pragma": "no-cache",
        "referer": "https://app.jisilu.cn/data/cbnew/",
        "sec-ch-ua": '" Not;A Brand";v="99", "Google Chrome";v="91", "Chromium";v="91"',
        "sec-ch-ua-mobile": "?0",
        "sec-fetch-dest": "empty",
        "sec-fetch-mode": "cors",
        "sec-fetch-site": "same-origin",
        "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/91.0.4472.164 Safari/537.36",
        "x-requested-with": "XMLHttpRequest",
    }
    params = {
        "___jsl": "LST___t=1627021692978",
    }
    payload = {
        "fprice": "",
        "tprice": "",
        "curr_iss_amt": "",
        "volume": "",
        "svolume": "",
        "premium_rt": "",
        "ytm_rt": "",
        "market": "",
        "rating_cd": "",
        "is_search": "N",
        "market_cd[]": "shmb",  # noqa: F601
        "market_cd[]": "shkc",  # noqa: F601
        "market_cd[]": "szmb",  # noqa: F601
        "market_cd[]": "szcy",  # noqa: F601
        "btype": "",
        "listed": "Y",
        "qflag": "N",
        "sw_cd": "",
        "bond_ids": "",
        "rp": "50",
    }
    r = requests.post(url, params=params, json=payload, headers=headers)
    data_json = r.json()
    temp_df = pd.DataFrame([item["cell"] for item in data_json["rows"]])
    temp_df.rename(
        columns={
            "bond_id": "代码",
            "bond_nm": "转债名称",
            "price": "现价",
            "increase_rt": "涨跌幅",
            "stock_id": "正股代码",
            "stock_nm": "正股名称",
            "sprice": "正股价",
            "sincrease_rt": "正股涨跌",
            "pb": "正股PB",
            "convert_price": "转股价",
            "convert_value": "转股价值",
            "premium_rt": "转股溢价率",
            "dblow": "双低",
            "rating_cd": "债券评级",
            "put_convert_price": "回售触发价",
            "force_redeem_price": "强赎触发价",
            "convert_amt_ratio": "转债占比",
            "maturity_dt": "到期时间",
            "year_left": "剩余年限",
            "curr_iss_amt": "剩余规模",
            "volume": "成交额",
            "turnover_rt": "换手率",
            "ytm_rt": "到期税前收益",
        },
        inplace=True,
    )

    temp_df = temp_df[
        [
            "代码",
            "转债名称",
            "现价",
            "涨跌幅",
            "正股代码",
            "正股名称",
            "正股价",
            "正股涨跌",
            "正股PB",
            "转股价",
            "转股价值",
            "转股溢价率",
            "债券评级",
            "回售触发价",
            "强赎触发价",
            "转债占比",
            "到期时间",
            "剩余年限",
            "剩余规模",
            "成交额",
            "换手率",
            "到期税前收益",
            "双低",
        ]
    ]
    temp_df["到期时间"] = pd.to_datetime(temp_df["到期时间"]).dt.date
    temp_df["现价"] = pd.to_numeric(temp_df["现价"], errors="coerce")
    temp_df["涨跌幅"] = pd.to_numeric(temp_df["涨跌幅"], errors="coerce")
    temp_df["正股价"] = pd.to_numeric(temp_df["正股价"], errors="coerce")
    temp_df["正股涨跌"] = pd.to_numeric(temp_df["正股涨跌"], errors="coerce")
    temp_df["正股PB"] = pd.to_numeric(temp_df["正股PB"], errors="coerce")
    temp_df["转股价"] = pd.to_numeric(temp_df["转股价"], errors="coerce")
    temp_df["转股价值"] = pd.to_numeric(temp_df["转股价值"], errors="coerce")
    temp_df["转股溢价率"] = pd.to_numeric(temp_df["转股溢价率"], errors="coerce")
    temp_df["回售触发价"] = pd.to_numeric(temp_df["回售触发价"], errors="coerce")
    temp_df["强赎触发价"] = pd.to_numeric(temp_df["强赎触发价"], errors="coerce")
    temp_df["转债占比"] = pd.to_numeric(temp_df["转债占比"], errors="coerce")
    temp_df["剩余年限"] = pd.to_numeric(temp_df["剩余年限"], errors="coerce")
    temp_df["剩余规模"] = pd.to_numeric(temp_df["剩余规模"], errors="coerce")
    temp_df["成交额"] = pd.to_numeric(temp_df["成交额"], errors="coerce")
    temp_df["换手率"] = pd.to_numeric(temp_df["换手率"], errors="coerce")
    temp_df["到期税前收益"] = pd.to_numeric(temp_df["到期税前收益"], errors="coerce")
    return temp_df


def bond_cb_redeem_jsl() -> pd.DataFrame:
    """
    集思录可转债-强赎
    https://www.jisilu.cn/data/cbnew/#redeem
    :return: 集思录可转债-强赎
    :rtype: pandas.DataFrame
    """
    url = "https://www.jisilu.cn/data/cbnew/redeem_list/"
    headers = {
        "Accept": "application/json, text/javascript, */*; q=0.01",
        "Accept-Encoding": "gzip, deflate, br",
        "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
        "Cache-Control": "no-cache",
        "Connection": "keep-alive",
        "Content-Length": "5",
        "Content-Type": "application/x-www-form-urlencoded; charset=UTF-8",
        "Host": "www.jisilu.cn",
        "Origin": "https://www.jisilu.cn",
        "Pragma": "no-cache",
        "Referer": "https://www.jisilu.cn/data/cbnew/",
        "sec-ch-ua": '" Not A;Brand";v="99", "Chromium";v="101", "Google Chrome";v="101"',
        "sec-ch-ua-mobile": "?0",
        "sec-ch-ua-platform": '"Windows"',
        "Sec-Fetch-Dest": "empty",
        "Sec-Fetch-Mode": "cors",
        "Sec-Fetch-Site": "same-origin",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/101.0.4951.67 Safari/537.36",
        "X-Requested-With": "XMLHttpRequest",
    }
    params = {
        "___jsl": "LST___t=1653394005966",
    }
    payload = {
        "rp": "50",
    }
    r = requests.post(url, params=params, json=payload, headers=headers)
    data_json = r.json()
    temp_df = pd.DataFrame([item["cell"] for item in data_json["rows"]])
    temp_df.rename(
        columns={
            "bond_id": "代码",
            "bond_nm": "名称",
            "price": "现价",
            "stock_id": "正股代码",
            "stock_nm": "正股名称",
            "margin_flg": "-",
            "btype": "-",
            "orig_iss_amt": "规模",
            "curr_iss_amt": "剩余规模",
            "convert_dt": "转股起始日",
            "convert_price": "转股价",
            "next_put_dt": "-",
            "redeem_dt": "-",
            "force_redeem": "-",
            "redeem_flag": "-",
            "redeem_price": "-",
            "redeem_price_ratio": "强赎触发比",
            "real_force_redeem_price": "强赎价",
            "redeem_remain_days": "-",
            "redeem_real_days": "-",
            "redeem_total_days": "-",
            "recount_dt": "-",
            "redeem_count_days": "-",
            "redeem_tc": "强赎条款",
            "sprice": "正股价",
            "delist_dt": "最后交易日",
            "maturity_dt": "到期日",
            "redeem_icon": "强赎状态",
            "redeem_orders": "-",
            "at_maturity": "-",
            "redeem_count": "强赎天计数",
            "after_next_put_dt": "-",
            "force_redeem_price": "强赎触发价",
        },
        inplace=True,
    )

    temp_df = temp_df[
        [
            "代码",
            "名称",
            "现价",
            "正股代码",
            "正股名称",
            "规模",
            "剩余规模",
            "转股起始日",
            "最后交易日",
            "到期日",
            "转股价",
            "强赎触发比",
            "强赎触发价",
            "正股价",
            "强赎价",
            "强赎天计数",
            "强赎条款",
            "强赎状态",
        ]
    ]
    temp_df["现价"] = pd.to_numeric(temp_df["现价"], errors="coerce")
    temp_df["规模"] = pd.to_numeric(temp_df["规模"], errors="coerce")
    temp_df["剩余规模"] = pd.to_numeric(temp_df["剩余规模"], errors="coerce")
    temp_df["转股起始日"] = pd.to_datetime(
        temp_df["转股起始日"], errors="coerce"
    ).dt.date
    temp_df["最后交易日"] = pd.to_datetime(
        temp_df["最后交易日"], errors="coerce"
    ).dt.date
    temp_df["到期日"] = pd.to_datetime(temp_df["到期日"], errors="coerce").dt.date
    temp_df["转股价"] = pd.to_numeric(temp_df["转股价"], errors="coerce")
    temp_df["强赎触发比"] = pd.to_numeric(
        temp_df["强赎触发比"].str.strip("%"), errors="coerce"
    )
    temp_df["强赎触发价"] = pd.to_numeric(temp_df["强赎触发价"], errors="coerce")
    temp_df["正股价"] = pd.to_numeric(temp_df["正股价"], errors="coerce")
    temp_df["强赎价"] = pd.to_numeric(temp_df["强赎价"], errors="coerce")
    temp_df["强赎天计数"] = temp_df["强赎天计数"].replace(
        r"^.*?(\d{1,2}\/\d{1,2} \| \d{1,2}).*?$", r"\1", regex=True
    )
    temp_df["强赎状态"] = temp_df["强赎状态"].map(
        {
            "R": "已公告强赎",
            "O": "公告要强赎",
            "G": "公告不强赎",
            "B": "已满足强赎条件",
            "": "",
        }
    )
    return temp_df


def bond_cb_adj_logs_jsl(symbol: str = "128013") -> pd.DataFrame:
    """
    集思录-可转债转股价-调整记录
    https://app.jisilu.cn/data/cbnew/#cb
    :param symbol: 可转债代码
    :type symbol: str
    :return: 转股价调整记录
    :rtype: pandas.DataFrame
    """
    url = f"https://www.jisilu.cn/data/cbnew/adj_logs/?bond_id={symbol}"
    r = requests.get(url)
    data_text = r.text
    if "</table>" not in data_text:
        # 1. 该可转债没有转股价调整记录，服务端返回文本 '暂无数据'
        # 2. 无效可转债代码，服务端返回 {"timestamp":1639565628,"isError":1,"msg":"无效代码格式"}
        # 以上两种情况，返回空的 DataFrame
        return pd.DataFrame()
    else:
        temp_df = pd.read_html(StringIO(data_text), parse_dates=True)[0]
        temp_df.columns = [item.replace(" ", "") for item in temp_df.columns]
        temp_df["下修前转股价"] = pd.to_numeric(
            temp_df["下修前转股价"], errors="coerce"
        )
        temp_df["下修后转股价"] = pd.to_numeric(
            temp_df["下修后转股价"], errors="coerce"
        )
        temp_df["下修底价"] = pd.to_numeric(temp_df["下修底价"], errors="coerce")
        temp_df["股东大会日"] = pd.to_datetime(
            temp_df["股东大会日"], format="%Y-%m-%d", errors="coerce"
        ).dt.date
        temp_df["新转股价生效日期"] = pd.to_datetime(
            temp_df["新转股价生效日期"], format="%Y-%m-%d", errors="coerce"
        ).dt.date
        return temp_df


if __name__ == "__main__":
    bond_cb_index_jsl_df = bond_cb_index_jsl()
    print(bond_cb_index_jsl_df)

    bond_cb_jsl_df = bond_cb_jsl(cookie="")
    print(bond_cb_jsl_df)

    bond_cb_redeem_jsl_df = bond_cb_redeem_jsl()
    print(bond_cb_redeem_jsl_df)

    bond_cb_adj_logs_jsl_df = bond_cb_adj_logs_jsl(symbol="128013")
    print(bond_cb_adj_logs_jsl_df)
